@@ -1,45 +1,64 @@
 """
 .. include:: ../README.md
+
 ----
 """
 
 from __future__ import annotations
 
+import logging
 import typing as t
 
 from sqlglot import expressions as exp
-from sqlglot.dialects import Dialect, Dialects
-from sqlglot.diff import diff
-from sqlglot.errors import ErrorLevel, ParseError, TokenError, UnsupportedError
-from sqlglot.expressions import Expression
-from sqlglot.expressions import alias_ as alias
-from sqlglot.expressions import (
-    and_,
-    column,
-    condition,
-    except_,
-    from_,
-    intersect,
-    maybe_parse,
-    not_,
-    or_,
-    select,
-    subquery,
+from sqlglot.dialects.dialect import Dialect as Dialect, Dialects as Dialects
+from sqlglot.diff import diff as diff
+from sqlglot.errors import (
+    ErrorLevel as ErrorLevel,
+    ParseError as ParseError,
+    TokenError as TokenError,
+    UnsupportedError as UnsupportedError,
 )
-from sqlglot.expressions import table_ as table
-from sqlglot.expressions import to_column, to_table, union
-from sqlglot.generator import Generator
-from sqlglot.parser import Parser
-from sqlglot.schema import MappingSchema, Schema
-from sqlglot.tokens import Tokenizer, TokenType
+from sqlglot.expressions import (
+    Expression as Expression,
+    alias_ as alias,
+    and_ as and_,
+    cast as cast,
+    column as column,
+    condition as condition,
+    except_ as except_,
+    from_ as from_,
+    func as func,
+    intersect as intersect,
+    maybe_parse as maybe_parse,
+    not_ as not_,
+    or_ as or_,
+    select as select,
+    subquery as subquery,
+    table_ as table,
+    to_column as to_column,
+    to_identifier as to_identifier,
+    to_table as to_table,
+    union as union,
+)
+from sqlglot.generator import Generator as Generator
+from sqlglot.parser import Parser as Parser
+from sqlglot.schema import MappingSchema as MappingSchema, Schema as Schema
+from sqlglot.tokens import Tokenizer as Tokenizer, TokenType as TokenType
 
 if t.TYPE_CHECKING:
-    from sqlglot.dialects.dialect import DialectType
+    from sqlglot._typing import E
+    from sqlglot.dialects.dialect import DialectType as DialectType
 
-    T = t.TypeVar("T", bound=Expression)
+logger = logging.getLogger("sqlglot")
 
 
-__version__ = "10.6.3"
+try:
+    from sqlglot._version import __version__, __version_tuple__
+except ImportError:
+    logger.error(
+        "Unable to set __version__, run `pip install -e .` or `python setup.py develop` first."
+    )
+
 
 pretty = False
 """Whether to format generated SQL by default."""
@@ -68,9 +87,9 @@ def parse(sql: str, read: DialectType = None, **opts) -> t.List[t.Optional[Expre
 def parse_one(
     sql: str,
     read: None = None,
-    into: t.Type[T] = ...,
+    into: t.Type[E] = ...,
     **opts,
-) -> T:
+) -> E:
     ...
 
 
@@ -78,9 +97,9 @@ def parse_one(
 def parse_one(
     sql: str,
     read: DialectType,
-    into: t.Type[T],
+    into: t.Type[E],
     **opts,
-) -> T:
+) -> E:
     ...
 
 
@@ -170,7 +189,7 @@ def transpile(
     Returns:
         The list of transpiled SQL statements.
     """
-    write = write or read if identity else write
+    write = (read if write is None else write) if identity else write
     return [
         Dialect.get_or_raise(write)().generate(expression, **opts)
         for expression in parse(sql, read, error_level=error_level)
