@@ -1,30 +1,39 @@
 from __future__ import annotations
 
-from sqlglot import exp, generator, parser, tokens
+import typing as t
+
+from sqlglot import exp, generator, parser, tokens, transforms
 from sqlglot.dialects.dialect import (
     Dialect,
+    arrow_json_extract_scalar_sql,
+    datestrtodate_sql,
+    format_time_lambda,
     locate_to_strposition,
+    max_or_greatest,
+    min_or_least,
     no_ilike_sql,
     no_paren_current_date_sql,
+    no_pivot_sql,
     no_tablesample_sql,
     no_trycast_sql,
+    parse_date_delta_with_interval,
+    rename_func,
     strposition_to_locate_sql,
 )
 from sqlglot.helper import seq_get
 from sqlglot.tokens import TokenType
 
 
-def _show_parser(*args, **kwargs):
-    def _parse(self):
+def _show_parser(*args: t.Any, **kwargs: t.Any) -> t.Callable[[MySQL.Parser], exp.Show]:
+    def _parse(self: MySQL.Parser) -> exp.Show:
         return self._parse_show_mysql(*args, **kwargs)
 
     return _parse
 
 
-def _date_trunc_sql(self, expression):
-    unit = expression.name.lower()
-
-    expr = self.sql(expression.expression)
+def _date_trunc_sql(self: generator.Generator, expression: exp.DateTrunc) -> str:
+    expr = self.sql(expression, "this")
+    unit = expression.text("unit")
 
     if unit == "day":
         return f"DATE({expr})"
@@ -42,23 +51,23 @@ def _date_trunc_sql(self, expression):
         concat = f"CONCAT(YEAR({expr}), ' 1 1')"
         date_format = "%Y %c %e"
     else:
-        self.unsupported("Unexpected interval unit: {unit}")
+        self.unsupported(f"Unexpected interval unit: {unit}")
         return f"DATE({expr})"
 
     return f"STR_TO_DATE({concat}, '{date_format}')"
 
 
-def _str_to_date(args):
+def _str_to_date(args: t.List) -> exp.StrToDate:
     date_format = MySQL.format_time(seq_get(args, 1))
     return exp.StrToDate(this=seq_get(args, 0), format=date_format)
 
 
-def _str_to_date_sql(self, expression):
+def _str_to_date_sql(self: generator.Generator, expression: exp.StrToDate | exp.StrToTime) -> str:
     date_format = self.format_time(expression)
     return f"STR_TO_DATE({self.sql(expression.this)}, {date_format})"
 
 
-def _trim_sql(self, expression):
+def _trim_sql(self: generator.Generator, expression: exp.Trim) -> str:
     target = self.sql(expression, "this")
     trim_type = self.sql(expression, "position")
     remove_chars = self.sql(expression, "expression")
@@ -73,24 +82,13 @@ def _trim_sql(self, expression):
     return f"TRIM({trim_type}{remove_chars}{from_part}{target})"
 
 
-def _date_add(expression_class):
-    def func(args):
-        interval = seq_get(args, 1)
-        return expression_class(
-            this=seq_get(args, 0),
-            expression=interval.this,
-            unit=exp.Literal.string(interval.text("unit").lower()),
-        )
-
-    return func
-
-
-def _date_add_sql(kind):
-    def func(self, expression):
+def _date_add_sql(kind: str) -> t.Callable[[generator.Generator, exp.DateAdd | exp.DateSub], str]:
+    def func(self: generator.Generator, expression: exp.DateAdd | exp.DateSub) -> str:
         this = self.sql(expression, "this")
         unit = expression.text("unit").upper() or "DAY"
-        expression = self.sql(expression, "expression")
-        return f"DATE_{kind}({this}, INTERVAL {expression} {unit})"
+        return (
+            f"DATE_{kind}({this}, {self.sql(exp.Interval(this=expression.expression, unit=unit))})"
+        )
 
     return func
 
@@ -111,24 +109,26 @@ class MySQL(Dialect):
         "%k": "%-H",
         "%l": "%-I",
         "%T": "%H:%M:%S",
+        "%W": "%a",
     }
 
     class Tokenizer(tokens.Tokenizer):
         QUOTES = ["'", '"']
         COMMENTS = ["--", "#", ("/*", "*/")]
         IDENTIFIERS = ["`"]
-        ESCAPES = ["'", "\\"]
+        STRING_ESCAPES = ["'", "\\"]
         BIT_STRINGS = [("b'", "'"), ("B'", "'"), ("0b", "")]
         HEX_STRINGS = [("x'", "'"), ("X'", "'"), ("0x", "")]
 
         KEYWORDS = {
             **tokens.Tokenizer.KEYWORDS,
-            "MEDIUMTEXT": TokenType.MEDIUMTEXT,
+            "CHARSET": TokenType.CHARACTER_SET,
+            "LONGBLOB": TokenType.LONGBLOB,
             "LONGTEXT": TokenType.LONGTEXT,
             "MEDIUMBLOB": TokenType.MEDIUMBLOB,
-            "LONGBLOB": TokenType.LONGBLOB,
-            "START": TokenType.BEGIN,
+            "MEDIUMTEXT": TokenType.MEDIUMTEXT,
             "SEPARATOR": TokenType.SEPARATOR,
+            "START": TokenType.BEGIN,
             "_ARMSCII8": TokenType.INTRODUCER,
             "_ASCII": TokenType.INTRODUCER,
             "_BIG5": TokenType.INTRODUCER,
@@ -175,25 +175,23 @@ class MySQL(Dialect):
             "@@": TokenType.SESSION_PARAMETER,
         }
 
-        COMMANDS = tokens.Tokenizer.COMMANDS - {TokenType.SET, TokenType.SHOW}
+        COMMANDS = tokens.Tokenizer.COMMANDS - {TokenType.SHOW}
 
     class Parser(parser.Parser):
-        FUNC_TOKENS = {*parser.Parser.FUNC_TOKENS, TokenType.SCHEMA}  # type: ignore
+        FUNC_TOKENS = {*parser.Parser.FUNC_TOKENS, TokenType.SCHEMA, TokenType.DATABASE}
 
         FUNCTIONS = {
-            **parser.Parser.FUNCTIONS,  # type: ignore
-            "DATE_ADD": _date_add(exp.DateAdd),
-            "DATE_SUB": _date_add(exp.DateSub),
-            "STR_TO_DATE": _str_to_date,
-            "LOCATE": locate_to_strposition,
+            **parser.Parser.FUNCTIONS,
+            "DATE_ADD": parse_date_delta_with_interval(exp.DateAdd),
+            "DATE_FORMAT": format_time_lambda(exp.TimeToStr, "mysql"),
+            "DATE_SUB": parse_date_delta_with_interval(exp.DateSub),
             "INSTR": lambda args: exp.StrPosition(substr=seq_get(args, 1), this=seq_get(args, 0)),
-            "LEFT": lambda args: exp.Substring(
-                this=seq_get(args, 0), start=exp.Literal.number(1), length=seq_get(args, 1)
-            ),
+            "LOCATE": locate_to_strposition,
+            "STR_TO_DATE": _str_to_date,
         }
 
         FUNCTION_PARSERS = {
-            **parser.Parser.FUNCTION_PARSERS,  # type: ignore
+            **parser.Parser.FUNCTION_PARSERS,
             "GROUP_CONCAT": lambda self: self.expression(
                 exp.GroupConcat,
                 this=self._parse_lambda(),
@@ -201,15 +199,9 @@ class MySQL(Dialect):
             ),
         }
 
-        PROPERTY_PARSERS = {
-            **parser.Parser.PROPERTY_PARSERS,  # type: ignore
-            "ENGINE": lambda self: self._parse_property_assignment(exp.EngineProperty),
-        }
-
         STATEMENT_PARSERS = {
-            **parser.Parser.STATEMENT_PARSERS,  # type: ignore
+            **parser.Parser.STATEMENT_PARSERS,
             TokenType.SHOW: lambda self: self._parse_show(),
-            TokenType.SET: lambda self: self._parse_set(),
         }
 
         SHOW_PARSERS = {
@@ -267,15 +259,12 @@ class MySQL(Dialect):
         }
 
         SET_PARSERS = {
-            "GLOBAL": lambda self: self._parse_set_item_assignment("GLOBAL"),
+            **parser.Parser.SET_PARSERS,
             "PERSIST": lambda self: self._parse_set_item_assignment("PERSIST"),
             "PERSIST_ONLY": lambda self: self._parse_set_item_assignment("PERSIST_ONLY"),
-            "SESSION": lambda self: self._parse_set_item_assignment("SESSION"),
-            "LOCAL": lambda self: self._parse_set_item_assignment("LOCAL"),
             "CHARACTER SET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
             "CHARSET": lambda self: self._parse_set_item_charset("CHARACTER SET"),
             "NAMES": lambda self: self._parse_set_item_names(),
-            "TRANSACTION": lambda self: self._parse_set_transaction(),
         }
 
         PROFILE_TYPES = {
@@ -290,16 +279,15 @@ class MySQL(Dialect):
             "SWAPS",
         }
 
-        TRANSACTION_CHARACTERISTICS = {
-            "ISOLATION LEVEL REPEATABLE READ",
-            "ISOLATION LEVEL READ COMMITTED",
-            "ISOLATION LEVEL READ UNCOMMITTED",
-            "ISOLATION LEVEL SERIALIZABLE",
-            "READ WRITE",
-            "READ ONLY",
-        }
+        LOG_DEFAULTS_TO_LN = True
 
-        def _parse_show_mysql(self, this, target=False, full=None, global_=None):
+        def _parse_show_mysql(
+            self,
+            this: str,
+            target: bool | str = False,
+            full: t.Optional[bool] = None,
+            global_: t.Optional[bool] = None,
+        ) -> exp.Show:
             if target:
                 if isinstance(target, str):
                     self._match_text_seq(target)
@@ -314,7 +302,13 @@ class MySQL(Dialect):
                 db = None
             else:
                 position = None
-                db = self._parse_id_var() if self._match_text_seq("FROM") else None
+                db = None
+
+                if self._match(TokenType.FROM):
+                    db = self._parse_id_var()
+                elif self._match(TokenType.DOT):
+                    db = target_id
+                    target_id = self._parse_id_var()
 
             channel = self._parse_id_var() if self._match_text_seq("FOR", "CHANNEL") else None
 
@@ -349,16 +343,12 @@ class MySQL(Dialect):
                 offset=offset,
                 limit=limit,
                 mutex=mutex,
-                **{"global": global_},
+                **{"global": global_},  # type: ignore
             )
 
-        def _parse_var_from_options(self, options):
-            for option in options:
-                if self._match_text_seq(*option.split(" ")):
-                    return exp.Var(this=option)
-            return None
-
-        def _parse_oldstyle_limit(self):
+        def _parse_oldstyle_limit(
+            self,
+        ) -> t.Tuple[t.Optional[exp.Expression], t.Optional[exp.Expression]]:
             limit = None
             offset = None
             if self._match_text_seq("LIMIT"):
@@ -368,47 +358,20 @@ class MySQL(Dialect):
                 elif len(parts) == 2:
                     limit = parts[1]
                     offset = parts[0]
+
             return offset, limit
 
-        def _default_parse_set_item(self):
-            return self._parse_set_item_assignment(kind=None)
-
-        def _parse_set_item_assignment(self, kind):
-            if kind in {"GLOBAL", "SESSION"} and self._match_text_seq("TRANSACTION"):
-                return self._parse_set_transaction(global_=kind == "GLOBAL")
-
-            left = self._parse_primary() or self._parse_id_var()
-            if not self._match(TokenType.EQ):
-                self.raise_error("Expected =")
-            right = self._parse_statement() or self._parse_id_var()
-
-            this = self.expression(
-                exp.EQ,
-                this=left,
-                expression=right,
-            )
-
-            return self.expression(
-                exp.SetItem,
-                this=this,
-                kind=kind,
-            )
-
-        def _parse_set_item_charset(self, kind):
+        def _parse_set_item_charset(self, kind: str) -> exp.Expression:
             this = self._parse_string() or self._parse_id_var()
+            return self.expression(exp.SetItem, this=this, kind=kind)
 
-            return self.expression(
-                exp.SetItem,
-                this=this,
-                kind=kind,
-            )
-
-        def _parse_set_item_names(self):
+        def _parse_set_item_names(self) -> exp.Expression:
             charset = self._parse_string() or self._parse_id_var()
             if self._match_text_seq("COLLATE"):
                 collate = self._parse_string() or self._parse_id_var()
             else:
                 collate = None
+
             return self.expression(
                 exp.SetItem,
                 this=charset,
@@ -416,39 +379,41 @@ class MySQL(Dialect):
                 kind="NAMES",
             )
 
-        def _parse_set_transaction(self, global_=False):
-            self._match_text_seq("TRANSACTION")
-            characteristics = self._parse_csv(
-                lambda: self._parse_var_from_options(self.TRANSACTION_CHARACTERISTICS)
-            )
-            return self.expression(
-                exp.SetItem,
-                expressions=characteristics,
-                kind="TRANSACTION",
-                **{"global": global_},
-            )
-
     class Generator(generator.Generator):
         LOCKING_READS_SUPPORTED = True
         NULL_ORDERING_SUPPORTED = False
+        JOIN_HINTS = False
+        TABLE_HINTS = False
 
         TRANSFORMS = {
-            **generator.Generator.TRANSFORMS,  # type: ignore
+            **generator.Generator.TRANSFORMS,
             exp.CurrentDate: no_paren_current_date_sql,
-            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
-            exp.ILike: no_ilike_sql,
-            exp.TableSample: no_tablesample_sql,
-            exp.TryCast: no_trycast_sql,
+            exp.DateDiff: lambda self, e: self.func("DATEDIFF", e.this, e.expression),
             exp.DateAdd: _date_add_sql("ADD"),
+            exp.DateStrToDate: datestrtodate_sql,
             exp.DateSub: _date_add_sql("SUB"),
             exp.DateTrunc: _date_trunc_sql,
+            exp.DayOfMonth: rename_func("DAYOFMONTH"),
+            exp.DayOfWeek: rename_func("DAYOFWEEK"),
+            exp.DayOfYear: rename_func("DAYOFYEAR"),
             exp.GroupConcat: lambda self, e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
-            exp.StrToDate: _str_to_date_sql,
-            exp.StrToTime: _str_to_date_sql,
-            exp.Trim: _trim_sql,
+            exp.ILike: no_ilike_sql,
+            exp.JSONExtractScalar: arrow_json_extract_scalar_sql,
+            exp.Max: max_or_greatest,
+            exp.Min: min_or_least,
             exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
             exp.NullSafeNEQ: lambda self, e: self.not_sql(self.binary(e, "<=>")),
+            exp.Pivot: no_pivot_sql,
+            exp.Select: transforms.preprocess([transforms.eliminate_distinct_on]),
             exp.StrPosition: strposition_to_locate_sql,
+            exp.StrToDate: _str_to_date_sql,
+            exp.StrToTime: _str_to_date_sql,
+            exp.TableSample: no_tablesample_sql,
+            exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
+            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
+            exp.Trim: _trim_sql,
+            exp.TryCast: no_trycast_sql,
+            exp.WeekOfYear: rename_func("WEEKOFYEAR"),
         }
 
         TYPE_MAPPING = generator.Generator.TYPE_MAPPING.copy()
@@ -457,7 +422,15 @@ class MySQL(Dialect):
         TYPE_MAPPING.pop(exp.DataType.Type.MEDIUMBLOB)
         TYPE_MAPPING.pop(exp.DataType.Type.LONGBLOB)
 
-        def show_sql(self, expression):
+        PROPERTIES_LOCATION = {
+            **generator.Generator.PROPERTIES_LOCATION,
+            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
+            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
+        }
+
+        LIMIT_FETCH = "LIMIT"
+
+        def show_sql(self, expression: exp.Show) -> str:
             this = f" {expression.name}"
             full = " FULL" if expression.args.get("full") else ""
             global_ = " GLOBAL" if expression.args.get("global") else ""
@@ -497,29 +470,16 @@ class MySQL(Dialect):
 
             return f"SHOW{full}{global_}{this}{target}{types}{db}{query}{log}{position}{channel}{mutex_or_status}{like}{where}{offset}{limit}"
 
-        def _prefixed_sql(self, prefix, expression, arg):
+        def _prefixed_sql(self, prefix: str, expression: exp.Expression, arg: str) -> str:
             sql = self.sql(expression, arg)
             if not sql:
                 return ""
             return f" {prefix} {sql}"
 
-        def _oldstyle_limit_sql(self, expression):
+        def _oldstyle_limit_sql(self, expression: exp.Show) -> str:
             limit = self.sql(expression, "limit")
             offset = self.sql(expression, "offset")
             if limit:
                 limit_offset = f"{offset}, {limit}" if offset else limit
                 return f" LIMIT {limit_offset}"
             return ""
-
-        def setitem_sql(self, expression):
-            kind = self.sql(expression, "kind")
-            kind = f"{kind} " if kind else ""
-            this = self.sql(expression, "this")
-            expressions = self.expressions(expression)
-            collate = self.sql(expression, "collate")
-            collate = f" COLLATE {collate}" if collate else ""
-            global_ = "GLOBAL " if expression.args.get("global") else ""
-            return f"{global_}{kind}{this}{expressions}{collate}"
-
-        def set_sql(self, expression):
-            return f"SET {self.expressions(expression)}"
