@@ -3,13 +3,22 @@ from __future__ import annotations
 import re
 import typing as t
 
-from sqlglot import exp, generator, parser, tokens
-from sqlglot.dialects.dialect import Dialect, parse_date_delta, rename_func
+from sqlglot import exp, generator, parser, tokens, transforms
+from sqlglot.dialects.dialect import (
+    Dialect,
+    max_or_greatest,
+    min_or_least,
+    parse_date_delta,
+    rename_func,
+)
 from sqlglot.expressions import DataType
 from sqlglot.helper import seq_get
 from sqlglot.time import format_time
 from sqlglot.tokens import TokenType
 
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E
+
 FULL_FORMAT_TIME_MAPPING = {
     "weekday": "%A",
     "dw": "%A",
@@ -44,13 +53,17 @@ DATE_FMT_RE = re.compile("([dD]{1,2})|([mM]{1,2})|([yY]{1,4})|([hH]{1,2})|([sS]{
 TRANSPILE_SAFE_NUMBER_FMT = {"N", "C"}
 
 
-def _format_time_lambda(exp_class, full_format_mapping=None, default=None):
-    def _format_time(args):
+def _format_time_lambda(
+    exp_class: t.Type[E], full_format_mapping: t.Optional[bool] = None
+) -> t.Callable[[t.List], E]:
+    def _format_time(args: t.List) -> E:
+        assert len(args) == 2
+
         return exp_class(
-            this=seq_get(args, 1),
+            this=args[1],
             format=exp.Literal.string(
                 format_time(
-                    seq_get(args, 0).name or (TSQL.time_format if default is True else default),
+                    args[0].name,
                     {**TSQL.time_mapping, **FULL_FORMAT_TIME_MAPPING}
                     if full_format_mapping
                     else TSQL.time_mapping,
@@ -61,13 +74,17 @@ def _format_time_lambda(exp_class, full_format_mapping=None, default=None):
     return _format_time
 
 
-def _parse_format(args):
-    fmt = seq_get(args, 1)
-    number_fmt = fmt.name in TRANSPILE_SAFE_NUMBER_FMT or not DATE_FMT_RE.search(fmt.this)
+def _parse_format(args: t.List) -> exp.Expression:
+    assert len(args) == 2
+
+    fmt = args[1]
+    number_fmt = fmt.name in TRANSPILE_SAFE_NUMBER_FMT or not DATE_FMT_RE.search(fmt.name)
+
     if number_fmt:
-        return exp.NumberToStr(this=seq_get(args, 0), format=fmt)
+        return exp.NumberToStr(this=args[0], format=fmt)
+
     return exp.TimeToStr(
-        this=seq_get(args, 0),
+        this=args[0],
         format=exp.Literal.string(
             format_time(fmt.name, TSQL.format_time_mapping)
             if len(fmt.name) == 1
@@ -76,7 +93,7 @@ def _parse_format(args):
     )
 
 
-def _parse_eomonth(args):
+def _parse_eomonth(args: t.List) -> exp.Expression:
     date = seq_get(args, 0)
     month_lag = seq_get(args, 1)
     unit = DATE_DELTA_INTERVAL.get("month")
@@ -90,39 +107,61 @@ def _parse_eomonth(args):
     return exp.LastDateOfMonth(this=exp.DateAdd(this=date, expression=month_lag, unit=unit))
 
 
-def generate_date_delta_with_unit_sql(self, e):
-    func = "DATEADD" if isinstance(e, exp.DateAdd) else "DATEDIFF"
-    return f"{func}({self.format_args(e.text('unit'), e.expression, e.this)})"
+def _parse_hashbytes(args: t.List) -> exp.Expression:
+    kind, data = args
+    kind = kind.name.upper() if kind.is_string else ""
+
+    if kind == "MD5":
+        args.pop(0)
+        return exp.MD5(this=data)
+    if kind in ("SHA", "SHA1"):
+        args.pop(0)
+        return exp.SHA(this=data)
+    if kind == "SHA2_256":
+        return exp.SHA2(this=data, length=exp.Literal.number(256))
+    if kind == "SHA2_512":
+        return exp.SHA2(this=data, length=exp.Literal.number(512))
+
+    return exp.func("HASHBYTES", *args)
 
 
-def _format_sql(self, e):
+def generate_date_delta_with_unit_sql(
+    self: generator.Generator, expression: exp.DateAdd | exp.DateDiff
+) -> str:
+    func = "DATEADD" if isinstance(expression, exp.DateAdd) else "DATEDIFF"
+    return self.func(func, expression.text("unit"), expression.expression, expression.this)
+
+
+def _format_sql(self: generator.Generator, expression: exp.NumberToStr | exp.TimeToStr) -> str:
     fmt = (
-        e.args["format"]
-        if isinstance(e, exp.NumberToStr)
-        else exp.Literal.string(format_time(e.text("format"), TSQL.inverse_time_mapping))
+        expression.args["format"]
+        if isinstance(expression, exp.NumberToStr)
+        else exp.Literal.string(
+            format_time(
+                expression.text("format"), t.cast(t.Dict[str, str], TSQL.inverse_time_mapping)
+            )
+        )
     )
-    return f"FORMAT({self.format_args(e.this, fmt)})"
+    return self.func("FORMAT", expression.this, fmt)
 
 
-def _string_agg_sql(self, e):
-    e = e.copy()
+def _string_agg_sql(self: generator.Generator, expression: exp.GroupConcat) -> str:
+    expression = expression.copy()
 
-    this = e.this
-    distinct = e.find(exp.Distinct)
+    this = expression.this
+    distinct = expression.find(exp.Distinct)
     if distinct:
         # exp.Distinct can appear below an exp.Order or an exp.GroupConcat expression
         self.unsupported("T-SQL STRING_AGG doesn't support DISTINCT.")
-        this = distinct.expressions[0]
-        distinct.pop()
+        this = distinct.pop().expressions[0]
 
     order = ""
-    if isinstance(e.this, exp.Order):
-        if e.this.this:
-            this = e.this.this
-            e.this.this.pop()
-        order = f" WITHIN GROUP ({self.sql(e.this)[1:]})"  # Order has a leading space
+    if isinstance(expression.this, exp.Order):
+        if expression.this.this:
+            this = expression.this.this.pop()
+        order = f" WITHIN GROUP ({self.sql(expression.this)[1:]})"  # Order has a leading space
 
-    separator = e.args.get("separator") or exp.Literal.string(",")
+    separator = expression.args.get("separator") or exp.Literal.string(",")
     return f"STRING_AGG({self.format_args(this, separator)}){order}"
 
 
@@ -238,12 +277,11 @@ class TSQL(Dialect):
 
     class Tokenizer(tokens.Tokenizer):
         IDENTIFIERS = ['"', ("[", "]")]
-
         QUOTES = ["'", '"']
+        HEX_STRINGS = [("0x", ""), ("0X", "")]
 
         KEYWORDS = {
             **tokens.Tokenizer.KEYWORDS,
-            "BIT": TokenType.BOOLEAN,
             "DATETIME2": TokenType.DATETIME,
             "DATETIMEOFFSET": TokenType.TIMESTAMPTZ,
             "DECLARE": TokenType.COMMAND,
@@ -263,34 +301,46 @@ class TSQL(Dialect):
             "UNIQUEIDENTIFIER": TokenType.UNIQUEIDENTIFIER,
             "VARCHAR(MAX)": TokenType.TEXT,
             "XML": TokenType.XML,
+            "SYSTEM_USER": TokenType.CURRENT_USER,
         }
 
         # TSQL allows @, # to appear as a variable/identifier prefix
         SINGLE_TOKENS = tokens.Tokenizer.SINGLE_TOKENS.copy()
-        SINGLE_TOKENS.pop("@")
         SINGLE_TOKENS.pop("#")
 
     class Parser(parser.Parser):
         FUNCTIONS = {
-            **parser.Parser.FUNCTIONS,  # type: ignore
+            **parser.Parser.FUNCTIONS,
             "CHARINDEX": lambda args: exp.StrPosition(
                 this=seq_get(args, 1),
                 substr=seq_get(args, 0),
                 position=seq_get(args, 2),
             ),
-            "ISNULL": exp.Coalesce.from_arg_list,
             "DATEADD": parse_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
             "DATEDIFF": parse_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
             "DATENAME": _format_time_lambda(exp.TimeToStr, full_format_mapping=True),
             "DATEPART": _format_time_lambda(exp.TimeToStr),
+            "EOMONTH": _parse_eomonth,
+            "FORMAT": _parse_format,
             "GETDATE": exp.CurrentTimestamp.from_arg_list,
-            "SYSDATETIME": exp.CurrentTimestamp.from_arg_list,
+            "HASHBYTES": _parse_hashbytes,
             "IIF": exp.If.from_arg_list,
+            "ISNULL": exp.Coalesce.from_arg_list,
+            "JSON_VALUE": exp.JSONExtractScalar.from_arg_list,
             "LEN": exp.Length.from_arg_list,
             "REPLICATE": exp.Repeat.from_arg_list,
-            "JSON_VALUE": exp.JSONExtractScalar.from_arg_list,
-            "FORMAT": _parse_format,
-            "EOMONTH": _parse_eomonth,
+            "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
+            "SYSDATETIME": exp.CurrentTimestamp.from_arg_list,
+            "SUSER_NAME": exp.CurrentUser.from_arg_list,
+            "SUSER_SNAME": exp.CurrentUser.from_arg_list,
+            "SYSTEM_USER": exp.CurrentUser.from_arg_list,
+        }
+
+        JOIN_HINTS = {
+            "LOOP",
+            "HASH",
+            "MERGE",
+            "REMOTE",
         }
 
         VAR_LENGTH_DATATYPES = {
@@ -300,16 +350,19 @@ class TSQL(Dialect):
             DataType.Type.NCHAR,
         }
 
-        RETURNS_TABLE_TOKENS = parser.Parser.ID_VAR_TOKENS - {  # type: ignore
+        RETURNS_TABLE_TOKENS = parser.Parser.ID_VAR_TOKENS - {
             TokenType.TABLE,
-            *parser.Parser.TYPE_TOKENS,  # type: ignore
+            *parser.Parser.TYPE_TOKENS,
         }
 
         STATEMENT_PARSERS = {
-            **parser.Parser.STATEMENT_PARSERS,  # type: ignore
+            **parser.Parser.STATEMENT_PARSERS,
             TokenType.END: lambda self: self._parse_command(),
         }
 
+        LOG_BASE_FIRST = False
+        LOG_DEFAULTS_TO_LN = True
+
         def _parse_system_time(self) -> t.Optional[exp.Expression]:
             if not self._match_text_seq("FOR", "SYSTEM_TIME"):
                 return None
@@ -342,7 +395,7 @@ class TSQL(Dialect):
 
             return system_time
 
-        def _parse_table_parts(self, schema: bool = False) -> exp.Expression:
+        def _parse_table_parts(self, schema: bool = False) -> exp.Table:
             table = super()._parse_table_parts(schema=schema)
             table.set("system_time", self._parse_system_time())
             return table
@@ -408,15 +461,14 @@ class TSQL(Dialect):
             ):
                 return this
 
-            expressions = self._parse_csv(self._parse_udf_kwarg)
+            expressions = self._parse_csv(self._parse_function_parameter)
             return self.expression(exp.UserDefinedFunction, this=this, expressions=expressions)
 
     class Generator(generator.Generator):
         LOCKING_READS_SUPPORTED = True
 
         TYPE_MAPPING = {
-            **generator.Generator.TYPE_MAPPING,  # type: ignore
-            exp.DataType.Type.BOOLEAN: "BIT",
+            **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.INT: "INTEGER",
             exp.DataType.Type.DECIMAL: "NUMERIC",
             exp.DataType.Type.DATETIME: "DATETIME2",
@@ -424,19 +476,37 @@ class TSQL(Dialect):
         }
 
         TRANSFORMS = {
-            **generator.Generator.TRANSFORMS,  # type: ignore
+            **generator.Generator.TRANSFORMS,
             exp.DateAdd: generate_date_delta_with_unit_sql,
             exp.DateDiff: generate_date_delta_with_unit_sql,
             exp.CurrentDate: rename_func("GETDATE"),
             exp.CurrentTimestamp: rename_func("GETDATE"),
-            exp.If: rename_func("IIF"),
-            exp.NumberToStr: _format_sql,
-            exp.TimeToStr: _format_sql,
             exp.GroupConcat: _string_agg_sql,
+            exp.If: rename_func("IIF"),
+            exp.Max: max_or_greatest,
+            exp.MD5: lambda self, e: self.func("HASHBYTES", exp.Literal.string("MD5"), e.this),
+            exp.Min: min_or_least,
+            exp.NumberToStr: _format_sql,
+            exp.Select: transforms.preprocess([transforms.eliminate_distinct_on]),
+            exp.SHA: lambda self, e: self.func("HASHBYTES", exp.Literal.string("SHA1"), e.this),
+            exp.SHA2: lambda self, e: self.func(
+                "HASHBYTES", exp.Literal.string(f"SHA2_{e.args.get('length', 256)}"), e.this
+            ),
+            exp.TimeToStr: _format_sql,
         }
 
         TRANSFORMS.pop(exp.ReturnsProperty)
 
+        PROPERTIES_LOCATION = {
+            **generator.Generator.PROPERTIES_LOCATION,
+            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
+        }
+
+        LIMIT_FETCH = "FETCH"
+
+        def offset_sql(self, expression: exp.Offset) -> str:
+            return f"{super().offset_sql(expression)} ROWS"
+
         def systemtime_sql(self, expression: exp.SystemTime) -> str:
             kind = expression.args["kind"]
             if kind == "ALL":
