@@ -9,14 +9,14 @@ from collections.abc import Collection
 from contextlib import contextmanager
 from copy import copy
 from enum import Enum
+from itertools import count
 
 if t.TYPE_CHECKING:
     from sqlglot import exp
+    from sqlglot._typing import E, T
+    from sqlglot.dialects.dialect import DialectType
     from sqlglot.expressions import Expression
 
-    T = t.TypeVar("T")
-    E = t.TypeVar("E", bound=Expression)
-
 CAMEL_CASE_PATTERN = re.compile("(?<!^)(?=[A-Z])")
 PYTHON_VERSION = sys.version_info[:2]
 logger = logging.getLogger("sqlglot")
@@ -25,7 +25,7 @@ logger = logging.getLogger("sqlglot")
 class AutoName(Enum):
     """This is used for creating enum classes where `auto()` is the string form of the corresponding value's name."""
 
-    def _generate_next_value_(name, _start, _count, _last_values):  # type: ignore
+    def _generate_next_value_(name, _start, _count, _last_values):
         return name
 
 
@@ -59,7 +59,7 @@ def ensure_list(value):
     """
     if value is None:
         return []
-    elif isinstance(value, (list, tuple)):
+    if isinstance(value, (list, tuple)):
         return list(value)
 
     return [value]
@@ -92,7 +92,7 @@ def ensure_collection(value):
     )
 
 
-def csv(*args, sep: str = ", ") -> str:
+def csv(*args: str, sep: str = ", ") -> str:
     """
     Formats any number of string arguments as CSV.
 
@@ -131,11 +131,16 @@ def subclasses(
     ]
 
 
-def apply_index_offset(expressions: t.List[t.Optional[E]], offset: int) -> t.List[t.Optional[E]]:
+def apply_index_offset(
+    this: exp.Expression,
+    expressions: t.List[t.Optional[E]],
+    offset: int,
+) -> t.List[t.Optional[E]]:
     """
     Applies an offset to a given integer literal expression.
 
     Args:
+        this: the target of the index
         expressions: the expression the offset will be applied to, wrapped in a list.
         offset: the offset that will be applied.
 
@@ -148,11 +153,28 @@ def apply_index_offset(expressions: t.List[t.Optional[E]], offset: int) -> t.Lis
 
     expression = expressions[0]
 
-    if expression and expression.is_int:
-        expression = expression.copy()
-        logger.warning("Applying array index offset (%s)", offset)
-        expression.args["this"] = str(int(expression.this) + offset)  # type: ignore
-        return [expression]
+    from sqlglot import exp
+    from sqlglot.optimizer.annotate_types import annotate_types
+    from sqlglot.optimizer.simplify import simplify
+
+    if not this.type:
+        annotate_types(this)
+
+    if t.cast(exp.DataType, this.type).this not in (
+        exp.DataType.Type.UNKNOWN,
+        exp.DataType.Type.ARRAY,
+    ):
+        return expressions
+
+    if expression:
+        if not expression.type:
+            annotate_types(expression)
+        if t.cast(exp.DataType, expression.type).this in exp.DataType.INTEGER_TYPES:
+            logger.warning("Applying array index offset (%s)", offset)
+            expression = simplify(
+                exp.Add(this=expression.copy(), expression=exp.Literal.number(offset))
+            )
+            return [expression]
 
     return expressions
 
@@ -162,9 +184,7 @@ def camel_to_snake_case(name: str) -> str:
     return CAMEL_CASE_PATTERN.sub("_", name).upper()
 
 
-def while_changing(
-    expression: t.Optional[Expression], func: t.Callable[[t.Optional[Expression]], E]
-) -> E:
+def while_changing(expression: Expression, func: t.Callable[[Expression], E]) -> E:
     """
     Applies a transformation to a given expression until a fix point is reached.
 
@@ -176,8 +196,13 @@ def while_changing(
         The transformed expression.
     """
     while True:
+        for n, *_ in reversed(tuple(expression.walk())):
+            n._hash = hash(n)
         start = hash(expression)
         expression = func(expression)
+
+        for n, *_ in expression.walk():
+            n._hash = None
         if start == hash(expression):
             break
     return expression
@@ -279,9 +304,18 @@ def find_new_name(taken: t.Collection[str], base: str) -> str:
     return new
 
 
+def name_sequence(prefix: str) -> t.Callable[[], str]:
+    """Returns a name generator given a prefix (e.g. a0, a1, a2, ... if the prefix is "a")."""
+    sequence = count()
+    return lambda: f"{prefix}{next(sequence)}"
+
+
 def object_to_dict(obj: t.Any, **kwargs) -> t.Dict:
     """Returns a dictionary created from an object's attributes."""
-    return {**{k: copy(v) for k, v in vars(obj).copy().items()}, **kwargs}
+    return {
+        **{k: v.copy() if hasattr(v, "copy") else copy(v) for k, v in vars(obj).items()},
+        **kwargs,
+    }
 
 
 def split_num_words(
@@ -356,15 +390,6 @@ def flatten(values: t.Iterable[t.Iterable[t.Any] | t.Any]) -> t.Iterator[t.Any]:
             yield value
 
 
-def count_params(function: t.Callable) -> int:
-    """
-    Returns the number of formal parameters expected by a function, without counting "self"
-    and "cls", in case of instance and class methods, respectively.
-    """
-    count = function.__code__.co_argcount
-    return count - 1 if inspect.ismethod(function) else count
-
-
 def dict_depth(d: t.Dict) -> int:
     """
     Get the nesting depth of a dictionary.
@@ -403,3 +428,31 @@ def first(it: t.Iterable[T]) -> T:
     Useful for sets.
     """
     return next(i for i in it)
+
+
+def case_sensitive(text: str, dialect: DialectType) -> bool:
+    """Checks if text contains any case sensitive characters depending on dialect."""
+    from sqlglot.dialects.dialect import RESOLVES_IDENTIFIERS_AS_UPPERCASE
+
+    unsafe = str.islower if dialect in RESOLVES_IDENTIFIERS_AS_UPPERCASE else str.isupper
+    return any(unsafe(char) for char in text)
+
+
+def should_identify(text: str, identify: str | bool, dialect: DialectType = None) -> bool:
+    """Checks if text should be identified given an identify option.
+
+    Args:
+        text: the text to check.
+        identify:
+            "always" or `True`: always returns true.
+            "safe": true if there is no uppercase or lowercase character in `text`, depending on `dialect`.
+        dialect: the dialect to use in order to decide whether a text should be identified.
+
+    Returns:
+        Whether or not a string should be identified.
+    """
+    if identify is True or identify == "always":
+        return True
+    if identify == "safe":
+        return not case_sensitive(text, dialect)
+    return False
