@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import itertools
 
 from sqlglot import exp
@@ -17,9 +19,7 @@ def canonicalize(expression: exp.Expression) -> exp.Expression:
     expression = add_text_to_concat(expression)
     expression = coerce_type(expression)
     expression = remove_redundant_casts(expression)
-
-    if isinstance(expression, exp.Identifier):
-        expression.set("quoted", True)
+    expression = ensure_bool_predicates(expression)
 
     return expression
 
@@ -52,13 +52,24 @@ def remove_redundant_casts(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
+def ensure_bool_predicates(expression: exp.Expression) -> exp.Expression:
+    if isinstance(expression, exp.Connector):
+        _replace_int_predicate(expression.left)
+        _replace_int_predicate(expression.right)
+
+    elif isinstance(expression, (exp.Where, exp.Having)):
+        _replace_int_predicate(expression.this)
+
+    return expression
+
+
 def _coerce_date(a: exp.Expression, b: exp.Expression) -> None:
     for a, b in itertools.permutations([a, b]):
         if (
             a.type
             and a.type.this == exp.DataType.Type.DATE
             and b.type
-            and b.type.this != exp.DataType.Type.DATE
+            and b.type.this not in (exp.DataType.Type.DATE, exp.DataType.Type.INTERVAL)
         ):
             _replace_cast(b, "date")
 
@@ -68,3 +79,8 @@ def _replace_cast(node: exp.Expression, to: str) -> None:
     cast = exp.Cast(this=node.copy(), to=data_type)
     cast.type = data_type
     node.replace(cast)
+
+
+def _replace_int_predicate(expression: exp.Expression) -> None:
+    if expression.type and expression.type.this in exp.DataType.INTEGER_TYPES:
+        expression.replace(exp.NEQ(this=expression.copy(), expression=exp.Literal.number(0)))
