@@ -2,8 +2,7 @@ import datetime
 import inspect
 import unittest
 
-from sqlglot import expressions as exp
-from sqlglot import parse_one
+from sqlglot import expressions as exp, parse_one
 from sqlglot.dataframe.sql import functions as SF
 from sqlglot.errors import ErrorLevel
 
@@ -808,14 +807,17 @@ class TestFunctions(unittest.TestCase):
         self.assertEqual("DATE_ADD(cola, 2)", col.sql())
         col_col_for_add = SF.date_add("cola", "colb")
         self.assertEqual("DATE_ADD(cola, colb)", col_col_for_add.sql())
+        current_date_add = SF.date_add(SF.current_date(), 5)
+        self.assertEqual("DATE_ADD(CURRENT_DATE, 5)", current_date_add.sql())
+        self.assertEqual("DATEADD(day, 5, CURRENT_DATE)", current_date_add.sql(dialect="snowflake"))
 
     def test_date_sub(self):
         col_str = SF.date_sub("cola", 2)
-        self.assertEqual("DATE_SUB(cola, 2)", col_str.sql())
+        self.assertEqual("DATE_ADD(cola, -2)", col_str.sql())
         col = SF.date_sub(SF.col("cola"), 2)
-        self.assertEqual("DATE_SUB(cola, 2)", col.sql())
+        self.assertEqual("DATE_ADD(cola, -2)", col.sql())
         col_col_for_add = SF.date_sub("cola", "colb")
-        self.assertEqual("DATE_SUB(cola, colb)", col_col_for_add.sql())
+        self.assertEqual("DATE_ADD(cola, colb * -1)", col_col_for_add.sql())
 
     def test_date_diff(self):
         col_str = SF.date_diff("cola", "colb")
@@ -958,9 +960,9 @@ class TestFunctions(unittest.TestCase):
 
     def test_sha1(self):
         col_str = SF.sha1("Spark")
-        self.assertEqual("SHA1('Spark')", col_str.sql())
+        self.assertEqual("SHA('Spark')", col_str.sql())
         col = SF.sha1(SF.col("cola"))
-        self.assertEqual("SHA1(cola)", col.sql())
+        self.assertEqual("SHA(cola)", col.sql())
 
     def test_sha2(self):
         col_str = SF.sha2("Spark", 256)
@@ -1152,17 +1154,17 @@ class TestFunctions(unittest.TestCase):
 
     def test_regexp_extract(self):
         col_str = SF.regexp_extract("cola", r"(\d+)-(\d+)", 1)
-        self.assertEqual("REGEXP_EXTRACT(cola, '(\\\\d+)-(\\\\d+)', 1)", col_str.sql())
+        self.assertEqual("REGEXP_EXTRACT(cola, '(\\d+)-(\\d+)', 1)", col_str.sql())
         col = SF.regexp_extract(SF.col("cola"), r"(\d+)-(\d+)", 1)
-        self.assertEqual("REGEXP_EXTRACT(cola, '(\\\\d+)-(\\\\d+)', 1)", col.sql())
+        self.assertEqual("REGEXP_EXTRACT(cola, '(\\d+)-(\\d+)', 1)", col.sql())
         col_no_idx = SF.regexp_extract(SF.col("cola"), r"(\d+)-(\d+)")
-        self.assertEqual("REGEXP_EXTRACT(cola, '(\\\\d+)-(\\\\d+)')", col_no_idx.sql())
+        self.assertEqual("REGEXP_EXTRACT(cola, '(\\d+)-(\\d+)')", col_no_idx.sql())
 
     def test_regexp_replace(self):
         col_str = SF.regexp_replace("cola", r"(\d+)", "--")
-        self.assertEqual("REGEXP_REPLACE(cola, '(\\\\d+)', '--')", col_str.sql())
+        self.assertEqual("REGEXP_REPLACE(cola, '(\\d+)', '--')", col_str.sql())
         col = SF.regexp_replace(SF.col("cola"), r"(\d+)", "--")
-        self.assertEqual("REGEXP_REPLACE(cola, '(\\\\d+)', '--')", col.sql())
+        self.assertEqual("REGEXP_REPLACE(cola, '(\\d+)', '--')", col.sql())
 
     def test_initcap(self):
         col_str = SF.initcap("cola")
