@@ -6,8 +6,68 @@ class TestBigQuery(Validator):
     dialect = "bigquery"
 
     def test_bigquery(self):
+        self.validate_identity("SELECT * FROM x-0.a")
+        self.validate_identity("SELECT * FROM pivot CROSS JOIN foo")
+        self.validate_identity("SAFE_CAST(x AS STRING)")
+        self.validate_identity("SELECT * FROM a-b-c.mydataset.mytable")
+        self.validate_identity("SELECT * FROM abc-def-ghi")
+        self.validate_identity("SELECT * FROM a-b-c")
+        self.validate_identity("SELECT * FROM my-table")
+        self.validate_identity("SELECT * FROM my-project.mydataset.mytable")
+        self.validate_identity("SELECT * FROM pro-ject_id.c.d CROSS JOIN foo-bar")
+        self.validate_identity("x <> ''")
+        self.validate_identity("DATE_TRUNC(col, WEEK(MONDAY))")
+        self.validate_identity("SELECT b'abc'")
+        self.validate_identity("""SELECT * FROM UNNEST(ARRAY<STRUCT<x INT64>>[1, 2])""")
+        self.validate_identity("SELECT AS STRUCT 1 AS a, 2 AS b")
+        self.validate_identity("SELECT DISTINCT AS STRUCT 1 AS a, 2 AS b")
+        self.validate_identity("SELECT AS VALUE STRUCT(1 AS a, 2 AS b)")
         self.validate_identity("SELECT STRUCT<ARRAY<STRING>>(['2023-01-17'])")
+        self.validate_identity("SELECT STRUCT<STRING>((SELECT a FROM b.c LIMIT 1)).*")
         self.validate_identity("SELECT * FROM q UNPIVOT(values FOR quarter IN (b, c))")
+        self.validate_identity("""CREATE TABLE x (a STRUCT<values ARRAY<INT64>>)""")
+        self.validate_identity("""CREATE TABLE x (a STRUCT<b STRING OPTIONS (description='b')>)""")
+        self.validate_identity(
+            """CREATE TABLE x (a STRING OPTIONS (description='x')) OPTIONS (table_expiration_days=1)"""
+        )
+        self.validate_identity(
+            "SELECT * FROM (SELECT * FROM `t`) AS a UNPIVOT((c) FOR c_name IN (v1, v2))"
+        )
+        self.validate_identity(
+            "CREATE TABLE IF NOT EXISTS foo AS SELECT * FROM bla EXCEPT DISTINCT (SELECT * FROM bar) LIMIT 0"
+        )
+
+        self.validate_all("SELECT 1 AS hash", write={"bigquery": "SELECT 1 AS `hash`"})
+        self.validate_all('x <> ""', write={"bigquery": "x <> ''"})
+        self.validate_all('x <> """"""', write={"bigquery": "x <> ''"})
+        self.validate_all("x <> ''''''", write={"bigquery": "x <> ''"})
+        self.validate_all(
+            "CREATE TEMP TABLE foo AS SELECT 1",
+            write={"bigquery": "CREATE TEMPORARY TABLE foo AS SELECT 1"},
+        )
+        self.validate_all(
+            "SELECT * FROM `SOME_PROJECT_ID.SOME_DATASET_ID.INFORMATION_SCHEMA.SOME_VIEW`",
+            write={
+                "bigquery": "SELECT * FROM SOME_PROJECT_ID.SOME_DATASET_ID.INFORMATION_SCHEMA.SOME_VIEW",
+            },
+        )
+        self.validate_all(
+            "SELECT * FROM `my-project.my-dataset.my-table`",
+            write={"bigquery": "SELECT * FROM `my-project`.`my-dataset`.`my-table`"},
+        )
+        self.validate_all("LEAST(x, y)", read={"sqlite": "MIN(x, y)"})
+        self.validate_all("CAST(x AS CHAR)", write={"bigquery": "CAST(x AS STRING)"})
+        self.validate_all("CAST(x AS NCHAR)", write={"bigquery": "CAST(x AS STRING)"})
+        self.validate_all("CAST(x AS NVARCHAR)", write={"bigquery": "CAST(x AS STRING)"})
+        self.validate_all("CAST(x AS TIMESTAMP)", write={"bigquery": "CAST(x AS DATETIME)"})
+        self.validate_all("CAST(x AS TIMESTAMPTZ)", write={"bigquery": "CAST(x AS TIMESTAMP)"})
+        self.validate_all("CAST(x AS RECORD)", write={"bigquery": "CAST(x AS STRUCT)"})
+        self.validate_all(
+            "SELECT ARRAY(SELECT AS STRUCT 1 a, 2 b)",
+            write={
+                "bigquery": "SELECT ARRAY(SELECT AS STRUCT 1 AS a, 2 AS b)",
+            },
+        )
         self.validate_all(
             "REGEXP_CONTAINS('foo', '.*')",
             read={"bigquery": "REGEXP_CONTAINS('foo', '.*')"},
@@ -33,33 +93,36 @@ class TestBigQuery(Validator):
                 "spark": "'x\\''",
             },
         )
+        with self.assertRaises(ValueError):
+            transpile("'\\'", read="bigquery")
+
+        self.validate_all(
+            "'\\\\'",
+            write={
+                "bigquery": r"'\\'",
+                "duckdb": r"'\\'",
+                "presto": r"'\\'",
+                "hive": r"'\\'",
+            },
+        )
         self.validate_all(
             r'r"""/\*.*\*/"""',
             write={
-                "bigquery": r"'/\\*.*\\*/'",
-                "duckdb": r"'/\*.*\*/'",
-                "presto": r"'/\*.*\*/'",
+                "bigquery": r"r'/\*.*\*/'",
+                "duckdb": r"'/\\*.*\\*/'",
+                "presto": r"'/\\*.*\\*/'",
                 "hive": r"'/\\*.*\\*/'",
                 "spark": r"'/\\*.*\\*/'",
             },
         )
         self.validate_all(
-            r"'\\'",
+            r'R"""/\*.*\*/"""',
             write={
-                "bigquery": r"'\\'",
-                "duckdb": r"'\'",
-                "presto": r"'\'",
-                "hive": r"'\\'",
-            },
-        )
-        self.validate_all(
-            R'R"""/\*.*\*/"""',
-            write={
-                "bigquery": R"'/\\*.*\\*/'",
-                "duckdb": R"'/\*.*\*/'",
-                "presto": R"'/\*.*\*/'",
-                "hive": R"'/\\*.*\\*/'",
-                "spark": R"'/\\*.*\\*/'",
+                "bigquery": r"r'/\*.*\*/'",
+                "duckdb": r"'/\\*.*\\*/'",
+                "presto": r"'/\\*.*\\*/'",
+                "hive": r"'/\\*.*\\*/'",
+                "spark": r"'/\\*.*\\*/'",
             },
         )
         self.validate_all(
@@ -72,6 +135,16 @@ class TestBigQuery(Validator):
                 "spark": "CAST(a AS LONG)",
             },
         )
+        self.validate_all(
+            "CAST(a AS BYTES)",
+            write={
+                "bigquery": "CAST(a AS BYTES)",
+                "duckdb": "CAST(a AS BLOB)",
+                "presto": "CAST(a AS VARBINARY)",
+                "hive": "CAST(a AS BINARY)",
+                "spark": "CAST(a AS BINARY)",
+            },
+        )
         self.validate_all(
             "CAST(a AS NUMERIC)",
             write={
@@ -148,7 +221,6 @@ class TestBigQuery(Validator):
             "current_datetime",
             write={
                 "bigquery": "CURRENT_DATETIME()",
-                "duckdb": "CURRENT_DATETIME()",
                 "presto": "CURRENT_DATETIME()",
                 "hive": "CURRENT_DATETIME()",
                 "spark": "CURRENT_DATETIME()",
@@ -158,7 +230,7 @@ class TestBigQuery(Validator):
             "current_time",
             write={
                 "bigquery": "CURRENT_TIME()",
-                "duckdb": "CURRENT_TIME()",
+                "duckdb": "CURRENT_TIME",
                 "presto": "CURRENT_TIME()",
                 "hive": "CURRENT_TIME()",
                 "spark": "CURRENT_TIME()",
@@ -168,7 +240,7 @@ class TestBigQuery(Validator):
             "current_timestamp",
             write={
                 "bigquery": "CURRENT_TIMESTAMP()",
-                "duckdb": "CURRENT_TIMESTAMP()",
+                "duckdb": "CURRENT_TIMESTAMP",
                 "postgres": "CURRENT_TIMESTAMP",
                 "presto": "CURRENT_TIMESTAMP",
                 "hive": "CURRENT_TIMESTAMP()",
@@ -179,7 +251,7 @@ class TestBigQuery(Validator):
             "current_timestamp()",
             write={
                 "bigquery": "CURRENT_TIMESTAMP()",
-                "duckdb": "CURRENT_TIMESTAMP()",
+                "duckdb": "CURRENT_TIMESTAMP",
                 "postgres": "CURRENT_TIMESTAMP",
                 "presto": "CURRENT_TIMESTAMP",
                 "hive": "CURRENT_TIMESTAMP()",
@@ -191,7 +263,7 @@ class TestBigQuery(Validator):
             "DIV(x, y)",
             write={
                 "bigquery": "DIV(x, y)",
-                "duckdb": "CAST(x / y AS INT)",
+                "duckdb": "x // y",
             },
         )
 
@@ -227,6 +299,12 @@ class TestBigQuery(Validator):
                 "spark": "CREATE TABLE db.example_table (col_a STRUCT<struct_col_a: LONG, struct_col_b: STRUCT<nested_col_a: STRING, nested_col_b: STRING>>)",
             },
         )
+        self.validate_all(
+            "CREATE TABLE db.example_table (x int) PARTITION BY x cluster by x",
+            write={
+                "bigquery": "CREATE TABLE db.example_table (x INT64) PARTITION BY x CLUSTER BY x",
+            },
+        )
         self.validate_all(
             "SELECT * FROM a WHERE b IN UNNEST([1, 2, 3])",
             write={
@@ -259,7 +337,8 @@ class TestBigQuery(Validator):
         self.validate_all(
             "DATE_SUB(CURRENT_DATE(), INTERVAL 1 DAY)",
             write={
-                "postgres": "CURRENT_DATE - INTERVAL '1' DAY",
+                "postgres": "CURRENT_DATE - INTERVAL '1 DAY'",
+                "bigquery": "DATE_SUB(CURRENT_DATE, INTERVAL 1 DAY)",
             },
         )
         self.validate_all(
@@ -268,8 +347,8 @@ class TestBigQuery(Validator):
                 "bigquery": "DATE_ADD(CURRENT_DATE, INTERVAL 1 DAY)",
                 "duckdb": "CURRENT_DATE + INTERVAL 1 DAY",
                 "mysql": "DATE_ADD(CURRENT_DATE, INTERVAL 1 DAY)",
-                "postgres": "CURRENT_DATE + INTERVAL '1' DAY",
-                "presto": "DATE_ADD(DAY, 1, CURRENT_DATE)",
+                "postgres": "CURRENT_DATE + INTERVAL '1 DAY'",
+                "presto": "DATE_ADD('DAY', 1, CURRENT_DATE)",
                 "hive": "DATE_ADD(CURRENT_DATE, 1)",
                 "spark": "DATE_ADD(CURRENT_DATE, 1)",
             },
@@ -310,6 +389,18 @@ class TestBigQuery(Validator):
                 "snowflake": "SELECT cola, colb FROM (VALUES (1, 'test')) AS tab(cola, colb)",
             },
         )
+        self.validate_all(
+            "SELECT cola, colb FROM (VALUES (1, 'test')) AS tab",
+            write={
+                "bigquery": "SELECT cola, colb FROM UNNEST([STRUCT(1 AS _c0, 'test' AS _c1)])",
+            },
+        )
+        self.validate_all(
+            "SELECT cola, colb FROM (VALUES (1, 'test'))",
+            write={
+                "bigquery": "SELECT cola, colb FROM UNNEST([STRUCT(1 AS _c0, 'test' AS _c1)])",
+            },
+        )
         self.validate_all(
             "SELECT cola, colb, colc FROM (VALUES (1, 'test', NULL)) AS tab(cola, colb, colc)",
             write={
@@ -324,15 +415,33 @@ class TestBigQuery(Validator):
                 "bigquery": "SELECT * FROM (SELECT a, b, c FROM test) PIVOT(SUM(b) AS d, COUNT(*) AS e FOR c IN ('x', 'y'))",
             },
         )
+        self.validate_all(
+            "SELECT REGEXP_EXTRACT(abc, 'pattern(group)') FROM table",
+            write={
+                "duckdb": "SELECT REGEXP_EXTRACT(abc, 'pattern(group)', 1) FROM table",
+            },
+        )
         self.validate_identity("BEGIN A B C D E F")
         self.validate_identity("BEGIN TRANSACTION")
         self.validate_identity("COMMIT TRANSACTION")
         self.validate_identity("ROLLBACK TRANSACTION")
+        self.validate_identity("CAST(x AS BIGNUMERIC)")
+
+        self.validate_identity("SELECT * FROM UNNEST([1]) WITH ORDINALITY")
+        self.validate_all(
+            "SELECT * FROM UNNEST([1]) WITH OFFSET",
+            write={"bigquery": "SELECT * FROM UNNEST([1]) WITH OFFSET AS offset"},
+        )
+        self.validate_all(
+            "SELECT * FROM UNNEST([1]) WITH OFFSET y",
+            write={"bigquery": "SELECT * FROM UNNEST([1]) WITH OFFSET AS y"},
+        )
 
     def test_user_defined_functions(self):
         self.validate_identity(
             "CREATE TEMPORARY FUNCTION a(x FLOAT64, y FLOAT64) RETURNS FLOAT64 NOT DETERMINISTIC LANGUAGE js AS 'return x*y;'"
         )
+        self.validate_identity("CREATE TEMPORARY FUNCTION udf(x ANY TYPE) AS (x)")
         self.validate_identity("CREATE TEMPORARY FUNCTION a(x FLOAT64, y FLOAT64) AS ((x + 4) / y)")
         self.validate_identity(
             "CREATE TABLE FUNCTION a(x INT64) RETURNS TABLE <q STRING, r INT64> AS SELECT s, t"
@@ -375,3 +484,27 @@ class TestBigQuery(Validator):
                 "bigquery": "INSERT INTO test (cola, colb) VALUES (CAST(7 AS STRING), CAST(14 AS STRING))",
             },
         )
+
+    def test_merge(self):
+        self.validate_all(
+            """
+            MERGE dataset.Inventory T
+            USING dataset.NewArrivals S ON FALSE
+            WHEN NOT MATCHED BY TARGET AND product LIKE '%a%'
+            THEN DELETE
+            WHEN NOT MATCHED BY SOURCE AND product LIKE '%b%'
+            THEN DELETE""",
+            write={
+                "bigquery": "MERGE INTO dataset.Inventory AS T USING dataset.NewArrivals AS S ON FALSE WHEN NOT MATCHED AND product LIKE '%a%' THEN DELETE WHEN NOT MATCHED BY SOURCE AND product LIKE '%b%' THEN DELETE",
+                "snowflake": "MERGE INTO dataset.Inventory AS T USING dataset.NewArrivals AS S ON FALSE WHEN NOT MATCHED AND product LIKE '%a%' THEN DELETE WHEN NOT MATCHED AND product LIKE '%b%' THEN DELETE",
+            },
+        )
+
+    def test_rename_table(self):
+        self.validate_all(
+            "ALTER TABLE db.t1 RENAME TO db.t2",
+            write={
+                "snowflake": "ALTER TABLE db.t1 RENAME TO db.t2",
+                "bigquery": "ALTER TABLE db.t1 RENAME TO t2",
+            },
+        )
