@@ -6,24 +6,58 @@ class TestMySQL(Validator):
     dialect = "mysql"
 
     def test_ddl(self):
+        self.validate_identity(
+            "INSERT INTO x VALUES (1, 'a', 2.0) ON DUPLICATE KEY UPDATE SET x.id = 1"
+        )
+
         self.validate_all(
             "CREATE TABLE z (a INT) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='x'",
             write={
                 "duckdb": "CREATE TABLE z (a INT)",
                 "mysql": "CREATE TABLE z (a INT) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='x'",
                 "spark": "CREATE TABLE z (a INT) COMMENT 'x'",
+                "sqlite": "CREATE TABLE z (a INTEGER)",
+            },
+        )
+        self.validate_all(
+            "CREATE TABLE t (c DATETIME DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP) DEFAULT CHARSET=utf8 ROW_FORMAT=DYNAMIC",
+            write={
+                "mysql": "CREATE TABLE t (c DATETIME DEFAULT CURRENT_TIMESTAMP() ON UPDATE CURRENT_TIMESTAMP()) DEFAULT CHARACTER SET=utf8 ROW_FORMAT=DYNAMIC",
+            },
+        )
+        self.validate_all(
+            "CREATE TABLE x (id int not null auto_increment, primary key (id))",
+            write={
+                "sqlite": "CREATE TABLE x (id INTEGER NOT NULL AUTOINCREMENT PRIMARY KEY)",
+            },
+        )
+        self.validate_all(
+            "CREATE TABLE x (id int not null auto_increment)",
+            write={
+                "sqlite": "CREATE TABLE x (id INTEGER NOT NULL)",
+            },
+        )
+        self.validate_all(
+            "CREATE TABLE `foo` (`id` char(36) NOT NULL DEFAULT (uuid()), PRIMARY KEY (`id`), UNIQUE KEY `id` (`id`))",
+            write={
+                "mysql": "CREATE TABLE `foo` (`id` CHAR(36) NOT NULL DEFAULT (UUID()), PRIMARY KEY (`id`), UNIQUE `id` (`id`))",
             },
         )
 
     def test_identity(self):
+        self.validate_identity("SELECT CURRENT_TIMESTAMP(6)")
+        self.validate_identity("x ->> '$.name'")
         self.validate_identity("SELECT CAST(`a`.`b` AS INT) FROM foo")
         self.validate_identity("SELECT TRIM(LEADING 'bla' FROM ' XXX ')")
         self.validate_identity("SELECT TRIM(TRAILING 'bla' FROM ' XXX ')")
         self.validate_identity("SELECT TRIM(BOTH 'bla' FROM ' XXX ')")
         self.validate_identity("SELECT TRIM('bla' FROM ' XXX ')")
         self.validate_identity("@@GLOBAL.max_connections")
-
         self.validate_identity("CREATE TABLE A LIKE B")
+        self.validate_identity("SELECT * FROM t1, t2 FOR SHARE OF t1, t2 SKIP LOCKED")
+        self.validate_identity(
+            "SELECT * FROM t1, t2, t3 FOR SHARE OF t1 NOWAIT FOR UPDATE OF t2, t3 SKIP LOCKED"
+        )
 
         # SET Commands
         self.validate_identity("SET @var_name = expr")
@@ -64,6 +98,7 @@ class TestMySQL(Validator):
         self.validate_identity("SET TRANSACTION READ ONLY")
         self.validate_identity("SET GLOBAL TRANSACTION ISOLATION LEVEL REPEATABLE READ, READ WRITE")
         self.validate_identity("SELECT SCHEMA()")
+        self.validate_identity("SELECT DATABASE()")
 
     def test_types(self):
         self.validate_all(
@@ -86,10 +121,18 @@ class TestMySQL(Validator):
         )
 
     def test_canonical_functions(self):
-        self.validate_identity("SELECT LEFT('str', 2)", "SELECT SUBSTRING('str', 1, 2)")
+        self.validate_identity("SELECT LEFT('str', 2)", "SELECT LEFT('str', 2)")
         self.validate_identity("SELECT INSTR('str', 'substr')", "SELECT LOCATE('substr', 'str')")
         self.validate_identity("SELECT UCASE('foo')", "SELECT UPPER('foo')")
         self.validate_identity("SELECT LCASE('foo')", "SELECT LOWER('foo')")
+        self.validate_identity(
+            "SELECT DAY_OF_MONTH('2023-01-01')", "SELECT DAYOFMONTH('2023-01-01')"
+        )
+        self.validate_identity("SELECT DAY_OF_WEEK('2023-01-01')", "SELECT DAYOFWEEK('2023-01-01')")
+        self.validate_identity("SELECT DAY_OF_YEAR('2023-01-01')", "SELECT DAYOFYEAR('2023-01-01')")
+        self.validate_identity(
+            "SELECT WEEK_OF_YEAR('2023-01-01')", "SELECT WEEKOFYEAR('2023-01-01')"
+        )
 
     def test_escape(self):
         self.validate_all(
@@ -130,53 +173,87 @@ class TestMySQL(Validator):
         )
 
     def test_hexadecimal_literal(self):
-        self.validate_all(
-            "SELECT 0xCC",
-            write={
-                "mysql": "SELECT x'CC'",
-                "sqlite": "SELECT x'CC'",
-                "spark": "SELECT X'CC'",
-                "trino": "SELECT X'CC'",
-                "bigquery": "SELECT 0xCC",
-                "oracle": "SELECT 204",
-            },
-        )
-        self.validate_all(
-            "SELECT X'1A'",
-            write={
-                "mysql": "SELECT x'1A'",
-            },
-        )
-        self.validate_all(
-            "SELECT 0xz",
-            write={
-                "mysql": "SELECT `0xz`",
-            },
-        )
+        write_CC = {
+            "bigquery": "SELECT 0xCC",
+            "clickhouse": "SELECT 0xCC",
+            "databricks": "SELECT 204",
+            "drill": "SELECT 204",
+            "duckdb": "SELECT 204",
+            "hive": "SELECT 204",
+            "mysql": "SELECT x'CC'",
+            "oracle": "SELECT 204",
+            "postgres": "SELECT x'CC'",
+            "presto": "SELECT 204",
+            "redshift": "SELECT 204",
+            "snowflake": "SELECT x'CC'",
+            "spark": "SELECT X'CC'",
+            "sqlite": "SELECT x'CC'",
+            "starrocks": "SELECT x'CC'",
+            "tableau": "SELECT 204",
+            "teradata": "SELECT 204",
+            "trino": "SELECT X'CC'",
+            "tsql": "SELECT 0xCC",
+        }
+        write_CC_with_leading_zeros = {
+            "bigquery": "SELECT 0x0000CC",
+            "clickhouse": "SELECT 0x0000CC",
+            "databricks": "SELECT 204",
+            "drill": "SELECT 204",
+            "duckdb": "SELECT 204",
+            "hive": "SELECT 204",
+            "mysql": "SELECT x'0000CC'",
+            "oracle": "SELECT 204",
+            "postgres": "SELECT x'0000CC'",
+            "presto": "SELECT 204",
+            "redshift": "SELECT 204",
+            "snowflake": "SELECT x'0000CC'",
+            "spark": "SELECT X'0000CC'",
+            "sqlite": "SELECT x'0000CC'",
+            "starrocks": "SELECT x'0000CC'",
+            "tableau": "SELECT 204",
+            "teradata": "SELECT 204",
+            "trino": "SELECT X'0000CC'",
+            "tsql": "SELECT 0x0000CC",
+        }
+
+        self.validate_all("SELECT X'1A'", write={"mysql": "SELECT x'1A'"})
+        self.validate_all("SELECT 0xz", write={"mysql": "SELECT `0xz`"})
+        self.validate_all("SELECT 0xCC", write=write_CC)
+        self.validate_all("SELECT 0xCC ", write=write_CC)
+        self.validate_all("SELECT x'CC'", write=write_CC)
+        self.validate_all("SELECT 0x0000CC", write=write_CC_with_leading_zeros)
+        self.validate_all("SELECT x'0000CC'", write=write_CC_with_leading_zeros)
 
     def test_bits_literal(self):
-        self.validate_all(
-            "SELECT 0b1011",
-            write={
-                "mysql": "SELECT b'1011'",
-                "postgres": "SELECT b'1011'",
-                "oracle": "SELECT 11",
-            },
-        )
-        self.validate_all(
-            "SELECT B'1011'",
-            write={
-                "mysql": "SELECT b'1011'",
-                "postgres": "SELECT b'1011'",
-                "oracle": "SELECT 11",
-            },
-        )
+        write_1011 = {
+            "bigquery": "SELECT 11",
+            "clickhouse": "SELECT 0b1011",
+            "databricks": "SELECT 11",
+            "drill": "SELECT 11",
+            "hive": "SELECT 11",
+            "mysql": "SELECT b'1011'",
+            "oracle": "SELECT 11",
+            "postgres": "SELECT b'1011'",
+            "presto": "SELECT 11",
+            "redshift": "SELECT 11",
+            "snowflake": "SELECT 11",
+            "spark": "SELECT 11",
+            "sqlite": "SELECT 11",
+            "mysql": "SELECT b'1011'",
+            "tableau": "SELECT 11",
+            "teradata": "SELECT 11",
+            "trino": "SELECT 11",
+            "tsql": "SELECT 11",
+        }
+
+        self.validate_all("SELECT 0b1011", write=write_1011)
+        self.validate_all("SELECT b'1011'", write=write_1011)
 
     def test_string_literals(self):
         self.validate_all(
             'SELECT "2021-01-01" + INTERVAL 1 MONTH',
             write={
-                "mysql": "SELECT '2021-01-01' + INTERVAL 1 MONTH",
+                "mysql": "SELECT '2021-01-01' + INTERVAL '1' MONTH",
             },
         )
 
@@ -194,7 +271,121 @@ class TestMySQL(Validator):
             },
         )
 
+    def test_match_against(self):
+        self.validate_all(
+            "MATCH(col1, col2, col3) AGAINST('abc')",
+            read={
+                "": "MATCH(col1, col2, col3) AGAINST('abc')",
+                "mysql": "MATCH(col1, col2, col3) AGAINST('abc')",
+            },
+            write={
+                "": "MATCH(col1, col2, col3) AGAINST('abc')",
+                "mysql": "MATCH(col1, col2, col3) AGAINST('abc')",
+            },
+        )
+        self.validate_all(
+            "MATCH(col1, col2) AGAINST('abc' IN NATURAL LANGUAGE MODE)",
+            write={"mysql": "MATCH(col1, col2) AGAINST('abc' IN NATURAL LANGUAGE MODE)"},
+        )
+        self.validate_all(
+            "MATCH(col1, col2) AGAINST('abc' IN NATURAL LANGUAGE MODE WITH QUERY EXPANSION)",
+            write={
+                "mysql": "MATCH(col1, col2) AGAINST('abc' IN NATURAL LANGUAGE MODE WITH QUERY EXPANSION)"
+            },
+        )
+        self.validate_all(
+            "MATCH(col1, col2) AGAINST('abc' IN BOOLEAN MODE)",
+            write={"mysql": "MATCH(col1, col2) AGAINST('abc' IN BOOLEAN MODE)"},
+        )
+        self.validate_all(
+            "MATCH(col1, col2) AGAINST('abc' WITH QUERY EXPANSION)",
+            write={"mysql": "MATCH(col1, col2) AGAINST('abc' WITH QUERY EXPANSION)"},
+        )
+        self.validate_all(
+            "MATCH(a.b) AGAINST('abc')",
+            write={"mysql": "MATCH(a.b) AGAINST('abc')"},
+        )
+
+    def test_date_format(self):
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15', '%Y')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15', '%Y')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15' AS TIMESTAMPNTZ), 'yyyy')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15', '%m')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15', '%m')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15' AS TIMESTAMPNTZ), 'mm')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15', '%d')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15', '%d')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15' AS TIMESTAMPNTZ), 'DD')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15', '%Y-%m-%d')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15', '%Y-%m-%d')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15' AS TIMESTAMPNTZ), 'yyyy-mm-DD')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15 22:23:34', '%H')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15 22:23:34', '%H')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15 22:23:34' AS TIMESTAMPNTZ), 'hh24')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2017-06-15', '%w')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2017-06-15', '%w')",
+                "snowflake": "SELECT TO_CHAR(CAST('2017-06-15' AS TIMESTAMPNTZ), 'dy')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2009-10-04 22:23:00', '%W %M %Y')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2009-10-04 22:23:00', '%W %M %Y')",
+                "snowflake": "SELECT TO_CHAR(CAST('2009-10-04 22:23:00' AS TIMESTAMPNTZ), 'DY mmmm yyyy')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('2007-10-04 22:23:00', '%H:%i:%s')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('2007-10-04 22:23:00', '%T')",
+                "snowflake": "SELECT TO_CHAR(CAST('2007-10-04 22:23:00' AS TIMESTAMPNTZ), 'hh24:mi:ss')",
+            },
+        )
+        self.validate_all(
+            "SELECT DATE_FORMAT('1900-10-04 22:23:00', '%d %y %a %d %m %b')",
+            write={
+                "mysql": "SELECT DATE_FORMAT('1900-10-04 22:23:00', '%d %y %W %d %m %b')",
+                "snowflake": "SELECT TO_CHAR(CAST('1900-10-04 22:23:00' AS TIMESTAMPNTZ), 'DD yy DY DD mm mon')",
+            },
+        )
+
+    def test_mysql_time(self):
+        self.validate_identity("FROM_UNIXTIME(a, b)")
+        self.validate_identity("FROM_UNIXTIME(a, b, c)")
+        self.validate_identity("TIME_STR_TO_UNIX(x)", "UNIX_TIMESTAMP(x)")
+
     def test_mysql(self):
+        self.validate_all(
+            "SELECT * FROM t LOCK IN SHARE MODE", write={"mysql": "SELECT * FROM t FOR SHARE"}
+        )
+        self.validate_all(
+            "SELECT DATE(DATE_SUB(`dt`, INTERVAL DAYOFMONTH(`dt`) - 1 DAY)) AS __timestamp FROM tableT",
+            write={
+                "mysql": "SELECT DATE(DATE_SUB(`dt`, INTERVAL (DAYOFMONTH(`dt`) - 1) DAY)) AS __timestamp FROM tableT",
+            },
+        )
         self.validate_all(
             "SELECT a FROM tbl FOR UPDATE",
             write={
@@ -202,6 +393,7 @@ class TestMySQL(Validator):
                 "mysql": "SELECT a FROM tbl FOR UPDATE",
                 "oracle": "SELECT a FROM tbl FOR UPDATE",
                 "postgres": "SELECT a FROM tbl FOR UPDATE",
+                "redshift": "SELECT a FROM tbl",
                 "tsql": "SELECT a FROM tbl FOR UPDATE",
             },
         )
@@ -249,26 +441,26 @@ class TestMySQL(Validator):
             "CREATE TABLE z (a INT) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='x'"
         )
         self.validate_identity(
-            "CREATE TABLE z (a INT DEFAULT NULL, PRIMARY KEY(a)) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='x'"
+            "CREATE TABLE z (a INT DEFAULT NULL, PRIMARY KEY (a)) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='x'"
         )
 
         self.validate_all(
             """
             CREATE TABLE `t_customer_account` (
-              "id" int(11) NOT NULL AUTO_INCREMENT,
-              "customer_id" int(11) DEFAULT NULL COMMENT '客户id',
-              "bank" varchar(100) COLLATE utf8_bin DEFAULT NULL COMMENT '行别',
-              "account_no" varchar(100) COLLATE utf8_bin DEFAULT NULL COMMENT '账号',
-              PRIMARY KEY ("id")
+              `id` int(11) NOT NULL AUTO_INCREMENT,
+              `customer_id` int(11) DEFAULT NULL COMMENT '客户id',
+              `bank` varchar(100) COLLATE utf8_bin DEFAULT NULL COMMENT '行别',
+              `account_no` varchar(100) COLLATE utf8_bin DEFAULT NULL COMMENT '账号',
+              PRIMARY KEY (`id`)
             ) ENGINE=InnoDB AUTO_INCREMENT=1 DEFAULT CHARACTER SET=utf8 COLLATE=utf8_bin COMMENT='客户账户表'
             """,
             write={
                 "mysql": """CREATE TABLE `t_customer_account` (
-  'id' INT(11) NOT NULL AUTO_INCREMENT,
-  'customer_id' INT(11) DEFAULT NULL COMMENT '客户id',
-  'bank' VARCHAR(100) COLLATE utf8_bin DEFAULT NULL COMMENT '行别',
-  'account_no' VARCHAR(100) COLLATE utf8_bin DEFAULT NULL COMMENT '账号',
-  PRIMARY KEY('id')
+  `id` INT(11) NOT NULL AUTO_INCREMENT,
+  `customer_id` INT(11) DEFAULT NULL COMMENT '客户id',
+  `bank` VARCHAR(100) COLLATE utf8_bin DEFAULT NULL COMMENT '行别',
+  `account_no` VARCHAR(100) COLLATE utf8_bin DEFAULT NULL COMMENT '账号',
+  PRIMARY KEY (`id`)
 )
 ENGINE=InnoDB
 AUTO_INCREMENT=1
@@ -415,6 +607,10 @@ COMMENT='客户账户表'"""
         show = self.validate_identity("SHOW INDEX FROM foo FROM bar")
         self.assertEqual(show.text("db"), "bar")
 
+        self.validate_all(
+            "SHOW INDEX FROM bar.foo", write={"mysql": "SHOW INDEX FROM foo FROM bar"}
+        )
+
     def test_show_db_like_or_where_sql(self):
         for key in [
             "OPEN TABLES",
