@@ -1,4 +1,4 @@
-from sqlglot import UnsupportedError
+from sqlglot import UnsupportedError, exp, parse_one
 from tests.dialects.test_dialect import Validator
 
 
@@ -6,6 +6,186 @@ class TestSnowflake(Validator):
     dialect = "snowflake"
 
     def test_snowflake(self):
+        self.validate_identity("INITCAP('iqamqinterestedqinqthisqtopic', 'q')")
+        self.validate_identity("CAST(x AS GEOMETRY)")
+        self.validate_identity("OBJECT_CONSTRUCT(*)")
+        self.validate_identity("SELECT TO_DATE('2019-02-28') + INTERVAL '1 day, 1 year'")
+        self.validate_identity("SELECT CAST('2021-01-01' AS DATE) + INTERVAL '1 DAY'")
+        self.validate_identity("SELECT HLL(*)")
+        self.validate_identity("SELECT HLL(a)")
+        self.validate_identity("SELECT HLL(DISTINCT t.a)")
+        self.validate_identity("SELECT HLL(a, b, c)")
+        self.validate_identity("SELECT HLL(DISTINCT a, b, c)")
+        self.validate_identity("$x")  # parameter
+        self.validate_identity("a$b")  # valid snowflake identifier
+        self.validate_identity("SELECT REGEXP_LIKE(a, b, c)")
+        self.validate_identity("PUT file:///dir/tmp.csv @%table")
+        self.validate_identity("CREATE TABLE foo (bar FLOAT AUTOINCREMENT START 0 INCREMENT 1)")
+        self.validate_identity("ALTER TABLE IF EXISTS foo SET TAG a = 'a', b = 'b', c = 'c'")
+        self.validate_identity("ALTER TABLE foo UNSET TAG a, b, c")
+        self.validate_identity(
+            'COPY INTO NEW_TABLE ("foo", "bar") FROM (SELECT $1, $2, $3, $4 FROM @%old_table)'
+        )
+        self.validate_identity("COMMENT IF EXISTS ON TABLE foo IS 'bar'")
+        self.validate_identity("SELECT CONVERT_TIMEZONE('UTC', 'America/Los_Angeles', col)")
+
+        self.validate_all("CAST(x AS CHAR VARYING)", write={"snowflake": "CAST(x AS VARCHAR)"})
+        self.validate_all("CAST(x AS CHARACTER VARYING)", write={"snowflake": "CAST(x AS VARCHAR)"})
+        self.validate_all("CAST(x AS NCHAR VARYING)", write={"snowflake": "CAST(x AS VARCHAR)"})
+        self.validate_all(
+            "OBJECT_CONSTRUCT(a, b, c, d)",
+            read={
+                "": "STRUCT(a as b, c as d)",
+            },
+            write={
+                "duckdb": "{'a': b, 'c': d}",
+                "snowflake": "OBJECT_CONSTRUCT(a, b, c, d)",
+            },
+        )
+        self.validate_all(
+            "SELECT i, p, o FROM qt QUALIFY ROW_NUMBER() OVER (PARTITION BY p ORDER BY o) = 1",
+            write={
+                "": "SELECT i, p, o FROM qt QUALIFY ROW_NUMBER() OVER (PARTITION BY p ORDER BY o NULLS LAST) = 1",
+                "databricks": "SELECT i, p, o FROM qt QUALIFY ROW_NUMBER() OVER (PARTITION BY p ORDER BY o NULLS LAST) = 1",
+                "hive": "SELECT i, p, o FROM (SELECT i, p, o, ROW_NUMBER() OVER (PARTITION BY p ORDER BY o NULLS LAST) AS _w FROM qt) AS _t WHERE _w = 1",
+                "presto": "SELECT i, p, o FROM (SELECT i, p, o, ROW_NUMBER() OVER (PARTITION BY p ORDER BY o) AS _w FROM qt) AS _t WHERE _w = 1",
+                "snowflake": "SELECT i, p, o FROM qt QUALIFY ROW_NUMBER() OVER (PARTITION BY p ORDER BY o) = 1",
+                "spark": "SELECT i, p, o FROM (SELECT i, p, o, ROW_NUMBER() OVER (PARTITION BY p ORDER BY o NULLS LAST) AS _w FROM qt) AS _t WHERE _w = 1",
+                "sqlite": "SELECT i, p, o FROM (SELECT i, p, o, ROW_NUMBER() OVER (PARTITION BY p ORDER BY o NULLS LAST) AS _w FROM qt) AS _t WHERE _w = 1",
+                "trino": "SELECT i, p, o FROM (SELECT i, p, o, ROW_NUMBER() OVER (PARTITION BY p ORDER BY o) AS _w FROM qt) AS _t WHERE _w = 1",
+            },
+        )
+        self.validate_all(
+            "SELECT BOOLOR_AGG(c1), BOOLOR_AGG(c2) FROM test",
+            write={
+                "": "SELECT LOGICAL_OR(c1), LOGICAL_OR(c2) FROM test",
+                "duckdb": "SELECT BOOL_OR(c1), BOOL_OR(c2) FROM test",
+                "postgres": "SELECT BOOL_OR(c1), BOOL_OR(c2) FROM test",
+                "snowflake": "SELECT BOOLOR_AGG(c1), BOOLOR_AGG(c2) FROM test",
+                "spark": "SELECT BOOL_OR(c1), BOOL_OR(c2) FROM test",
+                "sqlite": "SELECT MAX(c1), MAX(c2) FROM test",
+            },
+        )
+        self.validate_all(
+            "SELECT BOOLAND_AGG(c1), BOOLAND_AGG(c2) FROM test",
+            write={
+                "": "SELECT LOGICAL_AND(c1), LOGICAL_AND(c2) FROM test",
+                "duckdb": "SELECT BOOL_AND(c1), BOOL_AND(c2) FROM test",
+                "postgres": "SELECT BOOL_AND(c1), BOOL_AND(c2) FROM test",
+                "snowflake": "SELECT BOOLAND_AGG(c1), BOOLAND_AGG(c2) FROM test",
+                "spark": "SELECT BOOL_AND(c1), BOOL_AND(c2) FROM test",
+                "sqlite": "SELECT MIN(c1), MIN(c2) FROM test",
+            },
+        )
+        self.validate_all(
+            "TO_CHAR(x, y)",
+            read={
+                "": "TO_CHAR(x, y)",
+                "snowflake": "TO_VARCHAR(x, y)",
+            },
+            write={
+                "": "CAST(x AS TEXT)",
+                "databricks": "TO_CHAR(x, y)",
+                "drill": "TO_CHAR(x, y)",
+                "oracle": "TO_CHAR(x, y)",
+                "postgres": "TO_CHAR(x, y)",
+                "snowflake": "TO_CHAR(x, y)",
+                "teradata": "TO_CHAR(x, y)",
+            },
+        )
+        self.validate_all(
+            "SQUARE(x)",
+            write={
+                "bigquery": "POWER(x, 2)",
+                "clickhouse": "POWER(x, 2)",
+                "databricks": "POWER(x, 2)",
+                "drill": "POW(x, 2)",
+                "duckdb": "POWER(x, 2)",
+                "hive": "POWER(x, 2)",
+                "mysql": "POWER(x, 2)",
+                "oracle": "POWER(x, 2)",
+                "postgres": "x ^ 2",
+                "presto": "POWER(x, 2)",
+                "redshift": "POWER(x, 2)",
+                "snowflake": "POWER(x, 2)",
+                "spark": "POWER(x, 2)",
+                "sqlite": "POWER(x, 2)",
+                "starrocks": "POWER(x, 2)",
+                "trino": "POWER(x, 2)",
+                "tsql": "POWER(x, 2)",
+            },
+        )
+        self.validate_all(
+            "POWER(x, 2)",
+            read={
+                "oracle": "SQUARE(x)",
+                "snowflake": "SQUARE(x)",
+                "tsql": "SQUARE(x)",
+            },
+        )
+        self.validate_all(
+            "DIV0(foo, bar)",
+            write={
+                "snowflake": "IFF(bar = 0, 0, foo / bar)",
+                "sqlite": "CASE WHEN bar = 0 THEN 0 ELSE foo / bar END",
+                "presto": "IF(bar = 0, 0, foo / bar)",
+                "spark": "IF(bar = 0, 0, foo / bar)",
+                "hive": "IF(bar = 0, 0, foo / bar)",
+                "duckdb": "CASE WHEN bar = 0 THEN 0 ELSE foo / bar END",
+            },
+        )
+        self.validate_all(
+            "ZEROIFNULL(foo)",
+            write={
+                "snowflake": "IFF(foo IS NULL, 0, foo)",
+                "sqlite": "CASE WHEN foo IS NULL THEN 0 ELSE foo END",
+                "presto": "IF(foo IS NULL, 0, foo)",
+                "spark": "IF(foo IS NULL, 0, foo)",
+                "hive": "IF(foo IS NULL, 0, foo)",
+                "duckdb": "CASE WHEN foo IS NULL THEN 0 ELSE foo END",
+            },
+        )
+        self.validate_all(
+            "NULLIFZERO(foo)",
+            write={
+                "snowflake": "IFF(foo = 0, NULL, foo)",
+                "sqlite": "CASE WHEN foo = 0 THEN NULL ELSE foo END",
+                "presto": "IF(foo = 0, NULL, foo)",
+                "spark": "IF(foo = 0, NULL, foo)",
+                "hive": "IF(foo = 0, NULL, foo)",
+                "duckdb": "CASE WHEN foo = 0 THEN NULL ELSE foo END",
+            },
+        )
+        self.validate_all(
+            "CREATE OR REPLACE TEMPORARY TABLE x (y NUMBER IDENTITY(0, 1))",
+            write={
+                "snowflake": "CREATE OR REPLACE TEMPORARY TABLE x (y DECIMAL AUTOINCREMENT START 0 INCREMENT 1)",
+            },
+        )
+        self.validate_all(
+            "CREATE TEMPORARY TABLE x (y NUMBER AUTOINCREMENT(0, 1))",
+            write={
+                "snowflake": "CREATE TEMPORARY TABLE x (y DECIMAL AUTOINCREMENT START 0 INCREMENT 1)",
+            },
+        )
+        self.validate_all(
+            "CREATE TABLE x (y NUMBER IDENTITY START 0 INCREMENT 1)",
+            write={
+                "snowflake": "CREATE TABLE x (y DECIMAL AUTOINCREMENT START 0 INCREMENT 1)",
+            },
+        )
+        self.validate_all(
+            "ALTER TABLE foo ADD COLUMN id INT identity(1, 1)",
+            write={
+                "snowflake": "ALTER TABLE foo ADD COLUMN id INT AUTOINCREMENT START 1 INCREMENT 1",
+            },
+        )
+        self.validate_all(
+            "SELECT DAYOFWEEK('2016-01-02T23:39:20.123-07:00'::TIMESTAMP)",
+            write={
+                "snowflake": "SELECT DAYOFWEEK(CAST('2016-01-02T23:39:20.123-07:00' AS TIMESTAMPNTZ))",
+            },
+        )
         self.validate_all(
             "SELECT * FROM xxx WHERE col ilike '%Don''t%'",
             write={
@@ -25,9 +205,13 @@ class TestSnowflake(Validator):
             },
         )
         self.validate_all(
-            "SELECT * EXCLUDE a, b RENAME (c AS d, E as F) FROM xxx",
+            "SELECT * EXCLUDE (a, b) RENAME (c AS d, E AS F) FROM xxx",
+            read={
+                "duckdb": "SELECT * EXCLUDE (a, b) REPLACE (c AS d, E AS F) FROM xxx",
+            },
             write={
                 "snowflake": "SELECT * EXCLUDE (a, b) RENAME (c AS d, E AS F) FROM xxx",
+                "duckdb": "SELECT * EXCLUDE (a, b) REPLACE (c AS d, E AS F) FROM xxx",
             },
         )
         self.validate_all(
@@ -59,7 +243,7 @@ class TestSnowflake(Validator):
             write={
                 "bigquery": "SELECT UNIX_TO_TIME(1659981729)",
                 "snowflake": "SELECT TO_TIMESTAMP(1659981729)",
-                "spark": "SELECT FROM_UNIXTIME(1659981729)",
+                "spark": "SELECT CAST(FROM_UNIXTIME(1659981729) AS TIMESTAMP)",
             },
         )
         self.validate_all(
@@ -75,7 +259,7 @@ class TestSnowflake(Validator):
             write={
                 "bigquery": "SELECT UNIX_TO_TIME('1659981729')",
                 "snowflake": "SELECT TO_TIMESTAMP('1659981729')",
-                "spark": "SELECT FROM_UNIXTIME('1659981729')",
+                "spark": "SELECT CAST(FROM_UNIXTIME('1659981729') AS TIMESTAMP)",
             },
         )
         self.validate_all(
@@ -91,19 +275,18 @@ class TestSnowflake(Validator):
             write={
                 "bigquery": "SELECT PARSE_TIMESTAMP('%Y-%m-%d %H:%M:%S', '2013-04-05 01:02:03')",
                 "snowflake": "SELECT TO_TIMESTAMP('2013-04-05 01:02:03', 'yyyy-mm-dd hh24:mi:ss')",
-                "spark": "SELECT TO_TIMESTAMP('2013-04-05 01:02:03', 'yyyy-MM-dd HH:mm:ss')",
+                "spark": "SELECT TO_TIMESTAMP('2013-04-05 01:02:03', 'yyyy-MM-d HH:mm:ss')",
             },
         )
         self.validate_all(
-            "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'mm/dd/yyyy hh24:mi:ss')",
+            "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'mm/DD/yyyy hh24:mi:ss')",
             read={
                 "bigquery": "SELECT PARSE_TIMESTAMP('%m/%d/%Y %H:%M:%S', '04/05/2013 01:02:03')",
                 "duckdb": "SELECT STRPTIME('04/05/2013 01:02:03', '%m/%d/%Y %H:%M:%S')",
-                "snowflake": "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'mm/dd/yyyy hh24:mi:ss')",
             },
             write={
                 "bigquery": "SELECT PARSE_TIMESTAMP('%m/%d/%Y %H:%M:%S', '04/05/2013 01:02:03')",
-                "snowflake": "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'mm/dd/yyyy hh24:mi:ss')",
+                "snowflake": "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'mm/DD/yyyy hh24:mi:ss')",
                 "spark": "SELECT TO_TIMESTAMP('04/05/2013 01:02:03', 'MM/dd/yyyy HH:mm:ss')",
             },
         )
@@ -117,7 +300,7 @@ class TestSnowflake(Validator):
         self.validate_all(
             "SELECT fname, lname, age FROM person ORDER BY age DESC NULLS FIRST, fname ASC NULLS LAST, lname",
             write={
-                "duckdb": "SELECT fname, lname, age FROM person ORDER BY age DESC NULLS FIRST, fname NULLS LAST, lname NULLS LAST",
+                "duckdb": "SELECT fname, lname, age FROM person ORDER BY age DESC NULLS FIRST, fname, lname",
                 "postgres": "SELECT fname, lname, age FROM person ORDER BY age DESC, fname, lname",
                 "presto": "SELECT fname, lname, age FROM person ORDER BY age DESC NULLS FIRST, fname, lname",
                 "hive": "SELECT fname, lname, age FROM person ORDER BY age DESC NULLS FIRST, fname NULLS LAST, lname NULLS LAST",
@@ -132,6 +315,20 @@ class TestSnowflake(Validator):
                 "snowflake": "SELECT ARRAY_AGG(DISTINCT a)",
             },
         )
+        self.validate_all(
+            "ARRAY_TO_STRING(x, '')",
+            write={
+                "spark": "ARRAY_JOIN(x, '')",
+                "snowflake": "ARRAY_TO_STRING(x, '')",
+            },
+        )
+        self.validate_all(
+            "TO_ARRAY(x)",
+            write={
+                "spark": "ARRAY(x)",
+                "snowflake": "[x]",
+            },
+        )
         self.validate_all(
             "SELECT * FROM a INTERSECT ALL SELECT * FROM b",
             write={
@@ -165,22 +362,16 @@ class TestSnowflake(Validator):
         self.validate_all(
             r"SELECT $$a ' \ \t \x21 z $ $$",
             write={
-                "snowflake": r"SELECT 'a \' \\ \\t \\x21 z $ '",
+                "snowflake": r"SELECT 'a \' \ \t \x21 z $ '",
             },
         )
-        self.validate_identity("SELECT REGEXP_LIKE(a, b, c)")
+        self.validate_identity("REGEXP_REPLACE('target', 'pattern', '\n')")
         self.validate_all(
             "SELECT RLIKE(a, b)",
             write={
                 "snowflake": "SELECT REGEXP_LIKE(a, b)",
             },
         )
-        self.validate_all(
-            "SELECT a FROM test SAMPLE BLOCK (0.5) SEED (42)",
-            write={
-                "snowflake": "SELECT a FROM test TABLESAMPLE BLOCK (0.5) SEED (42)",
-            },
-        )
         self.validate_all(
             "SELECT a FROM test pivot",
             write={
@@ -209,14 +400,10 @@ class TestSnowflake(Validator):
             },
         )
         self.validate_all(
-            "DECODE(x, a, b, c, d)",
-            read={
-                "": "MATCHES(x, a, b, c, d)",
-            },
+            "DECODE(x, a, b, c, d, e)",
             write={
-                "": "MATCHES(x, a, b, c, d)",
-                "oracle": "DECODE(x, a, b, c, d)",
-                "snowflake": "DECODE(x, a, b, c, d)",
+                "": "CASE WHEN x = a OR (x IS NULL AND a IS NULL) THEN b WHEN x = c OR (x IS NULL AND c IS NULL) THEN d ELSE e END",
+                "snowflake": "CASE WHEN x = a OR (x IS NULL AND a IS NULL) THEN b WHEN x = c OR (x IS NULL AND c IS NULL) THEN d ELSE e END",
             },
         )
 
@@ -230,7 +417,7 @@ class TestSnowflake(Validator):
         self.validate_all(
             r"SELECT FIRST_VALUE(TABLE1.COLUMN1 RESPECT NULLS) OVER (PARTITION BY RANDOM_COLUMN1, RANDOM_COLUMN2 ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS MY_ALIAS FROM TABLE1",
             write={
-                "snowflake": r"SELECT FIRST_VALUE(TABLE1.COLUMN1) OVER (PARTITION BY RANDOM_COLUMN1, RANDOM_COLUMN2 ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS MY_ALIAS FROM TABLE1"
+                "snowflake": r"SELECT FIRST_VALUE(TABLE1.COLUMN1 RESPECT NULLS) OVER (PARTITION BY RANDOM_COLUMN1, RANDOM_COLUMN2 ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS MY_ALIAS FROM TABLE1"
             },
         )
         self.validate_all(
@@ -252,7 +439,44 @@ class TestSnowflake(Validator):
             },
         )
 
+    def test_sample(self):
+        self.validate_identity("SELECT * FROM testtable TABLESAMPLE BERNOULLI (20.3)")
+        self.validate_identity("SELECT * FROM testtable TABLESAMPLE (100)")
+        self.validate_identity("SELECT * FROM testtable TABLESAMPLE SYSTEM (3) SEED (82)")
+        self.validate_identity("SELECT * FROM testtable TABLESAMPLE (10 ROWS)")
+        self.validate_identity("SELECT * FROM testtable SAMPLE (10)")
+        self.validate_identity("SELECT * FROM testtable SAMPLE ROW (0)")
+        self.validate_identity("SELECT a FROM test SAMPLE BLOCK (0.5) SEED (42)")
+        self.validate_identity(
+            "SELECT i, j FROM table1 AS t1 INNER JOIN table2 AS t2 TABLESAMPLE (50) WHERE t2.j = t1.i"
+        )
+        self.validate_identity(
+            "SELECT * FROM (SELECT * FROM t1 JOIN t2 ON t1.a = t2.c) TABLESAMPLE (1)"
+        )
+
+        self.validate_all(
+            """
+            SELECT i, j
+                FROM
+                     table1 AS t1 SAMPLE (25)     -- 25% of rows in table1
+                         INNER JOIN
+                     table2 AS t2 SAMPLE (50)     -- 50% of rows in table2
+                WHERE t2.j = t1.i""",
+            write={
+                "snowflake": "SELECT i, j FROM table1 AS t1 SAMPLE (25) /* 25% of rows in table1 */ INNER JOIN table2 AS t2 SAMPLE (50) /* 50% of rows in table2 */ WHERE t2.j = t1.i",
+            },
+        )
+        self.validate_all(
+            "SELECT * FROM testtable SAMPLE BLOCK (0.012) REPEATABLE (99992)",
+            write={
+                "snowflake": "SELECT * FROM testtable SAMPLE BLOCK (0.012) SEED (99992)",
+            },
+        )
+
     def test_timestamps(self):
+        self.validate_identity("SELECT CAST('12:00:00' AS TIME)")
+        self.validate_identity("SELECT DATE_PART(month, a)")
+
         self.validate_all(
             "SELECT CAST(a AS TIMESTAMP)",
             write={
@@ -277,23 +501,22 @@ class TestSnowflake(Validator):
                 "snowflake": "SELECT CAST(a AS TIMESTAMPLTZ)",
             },
         )
-        self.validate_identity("SELECT EXTRACT(month FROM a)")
         self.validate_all(
             "SELECT EXTRACT('month', a)",
             write={
-                "snowflake": "SELECT EXTRACT('month' FROM a)",
+                "snowflake": "SELECT DATE_PART('month', a)",
             },
         )
         self.validate_all(
             "SELECT DATE_PART('month', a)",
             write={
-                "snowflake": "SELECT EXTRACT('month' FROM a)",
+                "snowflake": "SELECT DATE_PART('month', a)",
             },
         )
         self.validate_all(
             "SELECT DATE_PART(month, a::DATETIME)",
             write={
-                "snowflake": "SELECT EXTRACT(month FROM CAST(a AS DATETIME))",
+                "snowflake": "SELECT DATE_PART(month, CAST(a AS DATETIME))",
             },
         )
         self.validate_all(
@@ -310,9 +533,19 @@ class TestSnowflake(Validator):
                 "presto": "SELECT TO_UNIXTIME(CAST(foo AS TIMESTAMP)) * 1000 AS ddate FROM table_name",
             },
         )
+        self.validate_all(
+            "DATEADD(DAY, 5, CAST('2008-12-25' AS DATE))",
+            write={
+                "bigquery": "DATE_ADD(CAST('2008-12-25' AS DATE), INTERVAL 5 DAY)",
+                "snowflake": "DATEADD(DAY, 5, CAST('2008-12-25' AS DATE))",
+            },
+        )
+        self.validate_identity("DATEDIFF(DAY, 5, CAST('2008-12-25' AS DATE))")
 
     def test_semi_structured_types(self):
         self.validate_identity("SELECT CAST(a AS VARIANT)")
+        self.validate_identity("SELECT CAST(a AS ARRAY)")
+
         self.validate_all(
             "SELECT a::VARIANT",
             write={
@@ -320,7 +553,6 @@ class TestSnowflake(Validator):
                 "tsql": "SELECT CAST(a AS SQL_VARIANT)",
             },
         )
-        self.validate_identity("SELECT CAST(a AS ARRAY)")
         self.validate_all(
             "ARRAY_CONSTRUCT(0, 1, 2)",
             write={
@@ -339,10 +571,24 @@ class TestSnowflake(Validator):
         )
 
     def test_ddl(self):
+        self.validate_identity("CREATE TABLE geospatial_table (id INT, g GEOGRAPHY)")
+        self.validate_identity("CREATE MATERIALIZED VIEW a COMMENT='...' AS SELECT 1 FROM x")
+        self.validate_identity("CREATE DATABASE mytestdb_clone CLONE mytestdb")
+        self.validate_identity("CREATE SCHEMA mytestschema_clone CLONE testschema")
+        self.validate_identity("CREATE TABLE orders_clone CLONE orders")
+        self.validate_identity(
+            "CREATE TABLE orders_clone_restore CLONE orders AT (TIMESTAMP => TO_TIMESTAMP_TZ('04/05/2013 01:02:03', 'mm/dd/yyyy hh24:mi:ss'))"
+        )
+        self.validate_identity(
+            "CREATE TABLE orders_clone_restore CLONE orders BEFORE (STATEMENT => '8e5d0ca9-005e-44e6-b858-a8f5b37c5726')"
+        )
         self.validate_identity(
             "CREATE TABLE a (x DATE, y BIGINT) WITH (PARTITION BY (x), integration='q', auto_refresh=TRUE, file_format=(type = parquet))"
         )
-        self.validate_identity("CREATE MATERIALIZED VIEW a COMMENT='...' AS SELECT 1 FROM x")
+        self.validate_identity(
+            "CREATE SCHEMA mytestschema_clone_restore CLONE testschema BEFORE (TIMESTAMP => TO_TIMESTAMP(40 * 365 * 86400))"
+        )
+
         self.validate_all(
             "CREATE OR REPLACE TRANSIENT TABLE a (id INT)",
             read={
@@ -356,6 +602,12 @@ class TestSnowflake(Validator):
             },
         )
 
+        self.validate_all(
+            "CREATE TABLE a (b INT)",
+            read={"teradata": "CREATE MULTISET TABLE a (b INT)"},
+            write={"snowflake": "CREATE TABLE a (b INT)"},
+        )
+
     def test_user_defined_functions(self):
         self.validate_all(
             "CREATE FUNCTION a(x DATE, y BIGINT) RETURNS ARRAY LANGUAGE JAVASCRIPT AS $$ SELECT 1 $$",
@@ -536,12 +788,20 @@ FROM persons AS p, LATERAL FLATTEN(input => p.c, path => 'contact') AS f, LATERA
 
     def test_values(self):
         self.validate_all(
-            'SELECT c0, c1 FROM (VALUES (1, 2), (3, 4)) AS "t0"(c0, c1)',
+            'SELECT "c0", "c1" FROM (VALUES (1, 2), (3, 4)) AS "t0"("c0", "c1")',
             read={
                 "spark": "SELECT `c0`, `c1` FROM (VALUES (1, 2), (3, 4)) AS `t0`(`c0`, `c1`)",
             },
         )
 
+        self.validate_all(
+            """SELECT $1 AS "_1" FROM VALUES ('a'), ('b')""",
+            write={
+                "snowflake": """SELECT $1 AS "_1" FROM (VALUES ('a'), ('b'))""",
+                "spark": """SELECT @1 AS `_1` FROM VALUES ('a'), ('b')""",
+            },
+        )
+
     def test_describe_table(self):
         self.validate_all(
             "DESCRIBE TABLE db.table",
@@ -572,6 +832,14 @@ FROM persons AS p, LATERAL FLATTEN(input => p.c, path => 'contact') AS f, LATERA
             },
         )
 
+    def test_parse_like_any(self):
+        like = parse_one("a LIKE ANY fun('foo')", read="snowflake")
+        ilike = parse_one("a ILIKE ANY fun('foo')", read="snowflake")
+
+        self.assertIsInstance(like, exp.LikeAny)
+        self.assertIsInstance(ilike, exp.ILikeAny)
+        like.sql()  # check that this doesn't raise
+
     def test_match_recognize(self):
         for row in (
             "ONE ROW PER MATCH",
@@ -595,11 +863,13 @@ MATCH_RECOGNIZE (
   PARTITION BY a, b
   ORDER BY
     x DESC
-  MEASURES y AS b
+  MEASURES
+    y AS b
   {row}
   {after}
   PATTERN (^ S1 S2*? ( {{- S3 -}} S4 )+ | PERMUTE(S1, S2){{1,2}} $)
-  DEFINE x AS y
+  DEFINE
+    x AS y
 )""",
                     pretty=True,
                 )
