@@ -16,7 +16,20 @@ SUM(CASE WHEN x > 1 THEN 1 ELSE 0 END) / y
 'x'
 '\x'
 "x"
+'\z'
+'\\z'
+'\\\z'
+'\\\\z'
+'\\\\\z'
+'\\\\\\z'
+'\n'
+'\\n'
+'\\\n'
+'\\\\n'
+'\\\\\n'
+'\\\\\\n'
 ""
+"""x"""
 N'abc'
 x
 x % 1
@@ -61,6 +74,7 @@ a.b.INT(1.234)
 INT(x / 100)
 time * 100
 int * 100
+dec + 1
 x IN (-1, 1)
 x IN ('a', 'a''a')
 x IN ((1))
@@ -71,6 +85,7 @@ x IS TRUE
 x IS FALSE
 x IS TRUE IS TRUE
 x LIKE y IS TRUE
+TRIM('a' || 'b')
 MAP()
 GREATEST(x)
 LEAST(y)
@@ -86,7 +101,11 @@ CURRENT_DATE AT TIME ZONE zone_column
 CURRENT_DATE AT TIME ZONE 'UTC' AT TIME ZONE 'Asia/Tokio'
 ARRAY()
 ARRAY(1, 2)
+ARRAY(time, foo)
+ARRAY(foo, time)
+ARRAY(LENGTH(waiter_name) > 0)
 ARRAY_CONTAINS(x, 1)
+x.EXTRACT(1)
 EXTRACT(x FROM y)
 EXTRACT(DATE FROM y)
 EXTRACT(WEEK(monday) FROM created_at)
@@ -101,6 +120,7 @@ SPLIT(SPLIT(referrer, 'utm_source=')[OFFSET(1)], "&")[OFFSET(0)]
 x[ORDINAL(1)][SAFE_OFFSET(2)]
 x GLOB '??-*'
 x GLOB y
+ILIKE(x, 'z')
 x LIKE SUBSTR('abc', 1, 1)
 x LIKE y
 x LIKE a.y
@@ -111,10 +131,16 @@ x ILIKE '%y%' ESCAPE '\'
 1 AS escape
 INTERVAL '1' day
 INTERVAL '1' MONTH
-INTERVAL '1 day'
-INTERVAL 2 months
-INTERVAL 1 + 3 DAYS
+INTERVAL '-1' CURRENT_DATE
+INTERVAL '-31' CAST(GETDATE() AS DATE)
+INTERVAL (1 + 3) DAYS
+INTERVAL '1' day * 5
+5 * INTERVAL '1' day
+CASE WHEN TRUE THEN INTERVAL '15' days END
 CAST('45' AS INTERVAL DAYS)
+CAST(x AS UUID)
+FILTER(a, x -> x.a.b.c.d.e.f.g)
+FILTER(a, x -> FOO(x.a.b.c.d.e.f.g) + x.a.b.c.d.e.f.g)
 TIMESTAMP_DIFF(CURRENT_TIMESTAMP(), 1, DAY)
 DATETIME_DIFF(CURRENT_DATE, 1, DAY)
 QUANTILE(x, 0.5)
@@ -134,15 +160,23 @@ CASE WHEN SUM(x) > 3 THEN 1 END OVER (PARTITION BY x)
 SUM(ROW() OVER (PARTITION BY x))
 SUM(ROW() OVER (PARTITION BY x + 1))
 SUM(ROW() OVER (PARTITION BY x AND y))
+SUM(x) OVER (w ORDER BY y)
 (ROW() OVER ())
 CASE WHEN (x > 1) THEN 1 ELSE 0 END
 CASE (1) WHEN 1 THEN 1 ELSE 0 END
 CASE 1 WHEN 1 THEN 1 ELSE 0 END
+CASE 1 WHEN 1 THEN timestamp ELSE date END
 x AT TIME ZONE 'UTC'
 CAST('2025-11-20 00:00:00+00' AS TIMESTAMP) AT TIME ZONE 'Africa/Cairo'
 SET x = 1
 SET -v
 SET x = ';'
+SET variable = value
+SET GLOBAL variable = value
+SET LOCAL variable = value
+SET @user OFF
+@x
+@"x"
 COMMIT
 USE db
 USE role x
@@ -152,6 +186,7 @@ USE schema x.y
 NOT 1
 NOT NOT 1
 SELECT * FROM test
+SELECT * FROM db.FOO()
 SELECT *, 1 FROM test
 SELECT * FROM a.b
 SELECT * FROM a.b.c
@@ -176,6 +211,7 @@ SELECT DISTINCT TIMESTAMP_TRUNC(time_field, MONTH) AS time_value FROM "table"
 SELECT DISTINCT ON (x) x, y FROM z
 SELECT DISTINCT ON (x, y + 1) * FROM z
 SELECT DISTINCT ON (x.y) * FROM z
+SELECT DISTINCT FROM_SOMETHING
 SELECT top.x
 SELECT TIMESTAMP(DATE_TRUNC(DATE(time_field), MONTH)) AS time_value FROM "table"
 SELECT GREATEST((3 + 1), LEAST(3, 4))
@@ -185,6 +221,7 @@ SELECT COUNT(DISTINCT a, b)
 SELECT COUNT(DISTINCT a, b + 1)
 SELECT SUM(DISTINCT x)
 SELECT SUM(x IGNORE NULLS) AS x
+SELECT COUNT(x RESPECT NULLS)
 SELECT TRUNCATE(a, b)
 SELECT ARRAY_AGG(DISTINCT x IGNORE NULLS ORDER BY a, b DESC LIMIT 10) AS x
 SELECT ARRAY_AGG(STRUCT(x, x AS y) ORDER BY z DESC) AS x
@@ -221,9 +258,12 @@ SELECT JSON_EXTRACT(x, '$.name')
 SELECT JSON_EXTRACT_SCALAR(x, '$.name')
 SELECT x LIKE '%x%' FROM test
 SELECT * FROM test LIMIT 100
+SELECT * FROM test LIMIT 1 + 1
 SELECT * FROM test LIMIT 100 OFFSET 200
 SELECT * FROM test FETCH FIRST ROWS ONLY
 SELECT * FROM test FETCH FIRST 1 ROWS ONLY
+SELECT * FROM test ORDER BY id DESC FETCH FIRST 10 ROWS WITH TIES
+SELECT * FROM test ORDER BY id DESC FETCH FIRST 10 PERCENT ROWS WITH TIES
 SELECT * FROM test FETCH NEXT 1 ROWS ONLY
 SELECT (1 > 2) AS x FROM test
 SELECT NOT (1 > 2) FROM test
@@ -246,15 +286,12 @@ SELECT a FROM test WHERE TRUE OR NOT EXISTS(SELECT * FROM x)
 SELECT a AS any, b AS some, c AS all, d AS exists FROM test WHERE a = ANY (SELECT 1)
 SELECT a FROM test WHERE a > ALL (SELECT 1)
 SELECT a FROM test WHERE (a, b) IN (SELECT 1, 2)
+SELECT X((SELECT 1) UNION (SELECT 2))
 SELECT a FROM test ORDER BY a
 SELECT a FROM test ORDER BY a, b
 SELECT x FROM tests ORDER BY a DESC, b DESC, c
 SELECT a FROM test ORDER BY a > 1
 SELECT * FROM test ORDER BY DATE DESC, TIMESTAMP DESC
-SELECT * FROM test DISTRIBUTE BY y SORT BY x DESC ORDER BY l
-SELECT * FROM test CLUSTER BY y
-SELECT * FROM test CLUSTER BY y
-SELECT * FROM test WHERE RAND() <= 0.1 DISTRIBUTE BY RAND() SORT BY RAND()
 SELECT a, b FROM test GROUP BY 1
 SELECT a, b FROM test GROUP BY a
 SELECT a, b FROM test WHERE a = 1 GROUP BY a HAVING a = 2
@@ -279,21 +316,22 @@ SELECT CASE CASE x > 1 WHEN TRUE THEN 1 END WHEN 1 THEN 1 ELSE 2 END
 SELECT a FROM (SELECT a FROM test) AS x
 SELECT a FROM (SELECT a FROM (SELECT a FROM test) AS y) AS x
 SELECT a FROM test WHERE a IN (1, 2, 3) OR b BETWEEN 1 AND 4
-SELECT a FROM test AS x TABLESAMPLE(BUCKET 1 OUT OF 5)
-SELECT a FROM test TABLESAMPLE(BUCKET 1 OUT OF 5)
-SELECT a FROM test TABLESAMPLE(BUCKET 1 OUT OF 5 ON x)
-SELECT a FROM test TABLESAMPLE(BUCKET 1 OUT OF 5 ON RAND())
-SELECT a FROM test TABLESAMPLE(0.1 PERCENT)
-SELECT a FROM test TABLESAMPLE(100)
-SELECT a FROM test TABLESAMPLE(100 ROWS)
+SELECT a FROM test AS x TABLESAMPLE (BUCKET 1 OUT OF 5)
+SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5)
+SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5 ON x)
+SELECT a FROM test TABLESAMPLE (BUCKET 1 OUT OF 5 ON RAND())
+SELECT a FROM test TABLESAMPLE (0.1 PERCENT)
+SELECT a FROM test TABLESAMPLE (100)
+SELECT a FROM test TABLESAMPLE (100 ROWS)
 SELECT a FROM test TABLESAMPLE BERNOULLI (50)
 SELECT a FROM test TABLESAMPLE SYSTEM (75)
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q'))
+SELECT 1 FROM a.b.table1 AS t UNPIVOT((c3) FOR c4 IN (a, b))
 SELECT a FROM test PIVOT(SOMEAGG(x, y, z) FOR q IN (1))
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) PIVOT(MAX(b) FOR c IN ('d'))
 SELECT a FROM (SELECT a, b FROM test) PIVOT(SUM(x) FOR y IN ('z', 'q'))
 SELECT a FROM test UNPIVOT(x FOR y IN (z, q)) AS x
-SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) AS x TABLESAMPLE(0.1)
+SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) AS x TABLESAMPLE (0.1)
 SELECT a FROM test PIVOT(SUM(x) FOR y IN ('z', 'q')) UNPIVOT(x FOR y IN (z, q)) AS x
 SELECT ABS(a) FROM test
 SELECT AVG(a) FROM test
@@ -376,7 +414,7 @@ SELECT 1 AS delete, 2 AS alter
 SELECT * FROM (x)
 SELECT * FROM ((x))
 SELECT * FROM ((SELECT 1))
-SELECT * FROM (x LATERAL VIEW EXPLODE(y) JOIN foo)
+SELECT * FROM (x CROSS JOIN foo LATERAL VIEW EXPLODE(y))
 SELECT * FROM (SELECT 1) AS x
 SELECT * FROM (SELECT 1 UNION SELECT 2) AS x
 SELECT * FROM (SELECT 1 UNION ALL SELECT 2) AS x
@@ -393,6 +431,7 @@ SELECT * EXCEPT (a, b) REPLACE (a AS b, b AS C)
 SELECT * EXCEPT (a, b) REPLACE (a AS b, b AS C) FROM y
 SELECT a.* EXCEPT (a, b), b.* REPLACE (a AS b, b AS C)
 SELECT a.* EXCEPT (a, b), b.* REPLACE (a AS b, b AS C) FROM x
+SELECT A.* EXCEPT (A.COL_1) FROM TABLE_1 AS A
 SELECT zoo, animals FROM (VALUES ('oakland', ARRAY('a', 'b')), ('sf', ARRAY('b', 'c'))) AS t(zoo, animals)
 SELECT zoo, animals FROM UNNEST(ARRAY(STRUCT('oakland' AS zoo, ARRAY('a', 'b') AS animals), STRUCT('sf' AS zoo, ARRAY('b', 'c') AS animals))) AS t(zoo, animals)
 WITH a AS (SELECT 1) SELECT 1 UNION ALL SELECT 2
@@ -408,10 +447,6 @@ SELECT 1 UNION (SELECT 2)
 (SELECT 1) ORDER BY x LIMIT 1 OFFSET 1
 (SELECT 1 UNION SELECT 2) UNION (SELECT 2 UNION ALL SELECT 3)
 (SELECT 1 UNION SELECT 2) ORDER BY x LIMIT 1 OFFSET 1
-(SELECT 1 UNION SELECT 2) CLUSTER BY y DESC
-(SELECT 1 UNION SELECT 2) SORT BY z
-(SELECT 1 UNION SELECT 2) DISTRIBUTE BY z
-(SELECT 1 UNION SELECT 2) DISTRIBUTE BY z SORT BY x
 SELECT 1 UNION (SELECT 2) ORDER BY x
 (SELECT 1) UNION SELECT 2 ORDER BY x
 SELECT * FROM (((SELECT 1) UNION SELECT 2) ORDER BY x LIMIT 1 OFFSET 1)
@@ -448,6 +483,7 @@ SELECT SUM(x) OVER (PARTITION BY a RANGE BETWEEN 1 AND 3)
 SELECT SUM(x) OVER (PARTITION BY a RANGE BETWEEN 1 FOLLOWING AND 3)
 SELECT SUM(x) OVER (PARTITION BY a RANGE BETWEEN 1 FOLLOWING AND UNBOUNDED FOLLOWING)
 SELECT AVG(x) OVER (ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) FROM t
+SELECT SUM(x) OVER (PARTITION BY a ORDER BY date ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING)
 SELECT LISTAGG(x) WITHIN GROUP (ORDER BY x) AS y
 SELECT LISTAGG(x) WITHIN GROUP (ORDER BY x DESC)
 SELECT PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY x)
@@ -497,7 +533,7 @@ CREATE TABLE z (a INT(11) DEFAULT NULL COMMENT '客户id')
 CREATE TABLE z (a INT(11) NOT NULL DEFAULT 1)
 CREATE TABLE z (a INT(11) NOT NULL DEFAULT -1)
 CREATE TABLE z (a INT(11) NOT NULL COLLATE utf8_bin AUTO_INCREMENT)
-CREATE TABLE z (a INT, PRIMARY KEY(a))
+CREATE TABLE z (a INT, PRIMARY KEY (a))
 CREATE TABLE z WITH (FORMAT='parquet') AS SELECT 1
 CREATE TABLE z WITH (FORMAT='ORC', x='2') AS SELECT 1
 CREATE TABLE z WITH (TABLE_FORMAT='iceberg', FORMAT='parquet') AS SELECT 1
@@ -523,22 +559,23 @@ CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON UPDATE SET DEFAULT)
 CREATE TABLE asd AS SELECT asd FROM asd WITH NO DATA
 CREATE TABLE asd AS SELECT asd FROM asd WITH DATA
 CREATE TABLE products (x INT GENERATED BY DEFAULT AS IDENTITY)
+CREATE TABLE products (x INT GENERATED BY DEFAULT ON NULL AS IDENTITY)
 CREATE TABLE products (x INT GENERATED ALWAYS AS IDENTITY)
+CREATE TABLE konyvszerzo (szerzo_azon INT CONSTRAINT konyvszerzo_szerzo_fk REFERENCES szerzo)
 CREATE TABLE IF NOT EXISTS customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (INCREMENT BY 1))
-CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10 INCREMENT BY 1))
+CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10 INCREMENT BY 1 MINVALUE -1 MAXVALUE 1 NO CYCLE))
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10))
+CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (CYCLE))
 CREATE TABLE foo (baz_id INT REFERENCES baz(id) DEFERRABLE)
-CREATE TABLE a, FALLBACK, LOG, JOURNAL, CHECKSUM=DEFAULT, DEFAULT MERGEBLOCKRATIO, BLOCKCOMPRESSION=MANUAL (a INT)
-CREATE TABLE a, NO FALLBACK PROTECTION, NO LOG, NO JOURNAL, CHECKSUM=ON, NO MERGEBLOCKRATIO, BLOCKCOMPRESSION=ALWAYS (a INT)
-CREATE TABLE a, WITH JOURNAL TABLE=x.y.z, CHECKSUM=OFF, MERGEBLOCKRATIO=1, DATABLOCKSIZE=10 KBYTES (a INT)
-CREATE TABLE a, BEFORE JOURNAL, AFTER JOURNAL, FREESPACE=1, DEFAULT DATABLOCKSIZE, BLOCKCOMPRESSION=DEFAULT (a INT)
-CREATE TABLE a, DUAL JOURNAL, DUAL AFTER JOURNAL, MERGEBLOCKRATIO=1 PERCENT, DATABLOCKSIZE=10 KILOBYTES (a INT)
-CREATE TABLE a, DUAL BEFORE JOURNAL, LOCAL AFTER JOURNAL, MAXIMUM DATABLOCKSIZE, BLOCKCOMPRESSION=AUTOTEMP(c1 INT) (a INT)
+CREATE TABLE foo (baz CHAR(4) CHARACTER SET LATIN UPPERCASE NOT CASESPECIFIC COMPRESS 'a')
+CREATE TABLE foo (baz DATE FORMAT 'YYYY/MM/DD' TITLE 'title' INLINE LENGTH 1 COMPRESS ('a', 'b'))
+CREATE TABLE t (title TEXT)
+CREATE TABLE foo (baz INT, inline TEXT)
 CREATE SET GLOBAL TEMPORARY TABLE a, NO BEFORE JOURNAL, NO AFTER JOURNAL, MINIMUM DATABLOCKSIZE, BLOCKCOMPRESSION=NEVER (a INT)
-CREATE MULTISET VOLATILE TABLE a, NOT LOCAL AFTER JOURNAL, FREESPACE=1 PERCENT, DATABLOCKSIZE=10 BYTES, WITH NO CONCURRENT ISOLATED LOADING FOR ALL (a INT)
 CREATE ALGORITHM=UNDEFINED DEFINER=foo@% SQL SECURITY DEFINER VIEW a AS (SELECT a FROM b)
 CREATE TEMPORARY TABLE x AS SELECT a FROM d
 CREATE TEMPORARY TABLE IF NOT EXISTS x AS SELECT a FROM d
+CREATE TABLE a (b INT) ON COMMIT PRESERVE ROWS
 CREATE VIEW x AS SELECT a FROM b
 CREATE VIEW IF NOT EXISTS x AS SELECT a FROM b
 CREATE VIEW z (a, b COMMENT 'b', c COMMENT 'c') AS SELECT a, b, c FROM d
@@ -551,6 +588,7 @@ CREATE TEMPORARY VIEW x AS WITH y AS (SELECT 1) SELECT * FROM y
 CREATE MATERIALIZED VIEW x.y.z AS SELECT a FROM b
 CREATE VIEW z (a, b)
 CREATE VIEW z (a, b COMMENT 'b', c COMMENT 'c')
+CREATE VIEW z AS LOCKING ROW FOR ACCESS SELECT a FROM b
 CREATE TEMPORARY FUNCTION f
 CREATE TEMPORARY FUNCTION f AS 'g'
 CREATE FUNCTION f
@@ -558,13 +596,17 @@ CREATE FUNCTION f AS 'g'
 CREATE FUNCTION a(b INT, c VARCHAR) AS 'SELECT 1'
 CREATE FUNCTION a() LANGUAGE sql
 CREATE FUNCTION a() LANGUAGE sql RETURNS INT
+CREATE FUNCTION a.b(x INT) RETURNS INT AS RETURN x + 1
 CREATE FUNCTION a.b.c()
 CREATE INDEX abc ON t (a)
 CREATE INDEX abc ON t (a, b, b)
+CREATE INDEX abc ON t (a NULLS LAST)
 CREATE UNIQUE INDEX abc ON t (a, b, b)
 CREATE UNIQUE INDEX IF NOT EXISTS my_idx ON tbl (a, b)
 CREATE SCHEMA x
 CREATE SCHEMA IF NOT EXISTS y
+CREATE DATABASE x
+CREATE DATABASE IF NOT EXISTS y
 CREATE PROCEDURE IF NOT EXISTS a.b.c() AS 'DECLARE BEGIN; END'
 CREATE OR REPLACE STAGE
 DESCRIBE x
@@ -585,6 +627,7 @@ INSERT OVERWRITE TABLE a.b PARTITION(ds) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD') SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds, hour) SELECT x FROM y
 INSERT OVERWRITE TABLE a.b PARTITION(ds = 'YYYY-MM-DD', hour = 'hh') SELECT x FROM y
+ALTER SESSION SET STATEMENT_TIMEOUT_IN_SECONDS=3
 ALTER AGGREGATE bla(foo) OWNER TO CURRENT_USER
 ALTER RULE foo ON bla RENAME TO baz
 ALTER ROLE CURRENT_USER WITH REPLICATION
@@ -592,12 +635,14 @@ ALTER SEQUENCE IF EXISTS baz RESTART WITH boo
 ALTER TYPE electronic_mail RENAME TO email
 ALTER VIEW foo ALTER COLUMN bla SET DEFAULT 'NOT SET'
 ALTER DOMAIN foo VALIDATE CONSTRAINT bla
+ALTER schema doo
 ANALYZE a.y
 DELETE FROM x WHERE y > 1
 DELETE FROM y
 DELETE FROM event USING sales WHERE event.eventid = sales.eventid
 DELETE FROM event USING sales, USING bla WHERE event.eventid = sales.eventid
 DELETE FROM event USING sales AS s WHERE event.eventid = s.eventid
+DELETE FROM event AS event USING sales AS s WHERE event.eventid = s.eventid
 PREPARE statement
 EXECUTE statement
 DROP TABLE a
@@ -605,6 +650,8 @@ DROP TABLE a.b
 DROP TABLE IF EXISTS a
 DROP TABLE IF EXISTS a.b
 DROP TABLE a CASCADE
+DROP TABLE s_hajo CASCADE CONSTRAINTS
+DROP TABLE a PURGE
 DROP VIEW a
 DROP VIEW a.b
 DROP VIEW IF EXISTS a
@@ -623,11 +670,13 @@ INSERT INTO x VALUES (1, 'a', 2.0)
 INSERT INTO x VALUES (1, 'a', 2.0), (1, 'a', 3.0), (X(), y[1], z.x)
 INSERT INTO y (a, b, c) SELECT a, b, c FROM x
 INSERT INTO y (SELECT 1) UNION (SELECT 2)
+INSERT INTO result_table (WITH test AS (SELECT * FROM source_table) SELECT * FROM test)
 INSERT OVERWRITE TABLE x IF EXISTS SELECT * FROM y
 INSERT OVERWRITE TABLE a.b IF EXISTS SELECT * FROM y
 INSERT OVERWRITE DIRECTORY 'x' SELECT 1
 INSERT OVERWRITE LOCAL DIRECTORY 'x' SELECT 1
 INSERT OVERWRITE LOCAL DIRECTORY 'x' ROW FORMAT DELIMITED FIELDS TERMINATED BY '1' COLLECTION ITEMS TERMINATED BY '2' MAP KEYS TERMINATED BY '3' LINES TERMINATED BY '4' NULL DEFINED AS '5' SELECT 1
+LOAD foo
 LOAD DATA INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy')
 LOAD DATA LOCAL INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy')
 LOAD DATA LOCAL INPATH 'x' INTO TABLE y PARTITION(ds = 'yyyy') INPUTFORMAT 'y'
@@ -644,6 +693,11 @@ UPDATE db.tbl_name SET foo = 123, foo_1 = 234 WHERE tbl_name.bar = 234
 TRUNCATE TABLE x
 OPTIMIZE TABLE y
 VACUUM FREEZE my_table
+COMMENT ON ACCESS METHOD gin IS 'GIN index access method'
+COMMENT ON COLUMN my_schema.my_table.my_column IS 'Employee ID number'
+COMMENT ON DATABASE my_database IS 'Development Database'
+COMMENT ON PROCEDURE my_proc(integer, integer) IS 'Runs a report'
+COMMENT ON TABLE my_schema.my_table IS 'Employee Information'
 WITH a AS (SELECT 1) INSERT INTO b SELECT * FROM a
 WITH a AS (SELECT * FROM b) UPDATE a SET col = 1
 WITH a AS (SELECT * FROM b) CREATE TABLE b AS SELECT * FROM a
@@ -651,7 +705,7 @@ WITH a AS (SELECT * FROM b) DELETE FROM a
 WITH a AS (SELECT * FROM b) CACHE TABLE a
 SELECT ? AS ? FROM x WHERE b BETWEEN ? AND ? GROUP BY ?, 1 LIMIT ?
 SELECT :hello, ? FROM x LIMIT :my_limit
-SELECT * FROM x FETCH NEXT @take ROWS ONLY OFFSET @skip
+SELECT * FROM x OFFSET @skip FETCH NEXT @take ROWS ONLY
 WITH a AS ((SELECT b.foo AS foo, b.bar AS bar FROM b) UNION ALL (SELECT c.foo AS foo, c.bar AS bar FROM c)) SELECT * FROM a
 WITH a AS ((SELECT 1 AS b) UNION ALL (SELECT 1 AS b)) SELECT * FROM a
 SELECT (WITH x AS (SELECT 1 AS y) SELECT * FROM x) AS z
@@ -659,31 +713,37 @@ SELECT ((SELECT 1) + 1)
 SELECT * FROM project.dataset.INFORMATION_SCHEMA.TABLES
 SELECT * FROM (table1 AS t1 LEFT JOIN table2 AS t2 ON 1 = 1)
 SELECT * FROM (tbl1 LEFT JOIN tbl2 ON 1 = 1)
-SELECT * FROM (tbl1 JOIN tbl2 JOIN tbl3)
-SELECT * FROM (tbl1 JOIN (tbl2 JOIN tbl3) ON bla = foo)
-SELECT * FROM (tbl1 JOIN LATERAL (SELECT * FROM bla) AS tbl)
+SELECT * FROM (tbl1, tbl2 JOIN tbl3 ON TRUE)
+SELECT * FROM (tbl1 JOIN (tbl2 CROSS JOIN tbl3) ON bla = foo)
+SELECT * FROM (tbl1, LATERAL (SELECT * FROM bla) AS tbl)
 SELECT CAST(x AS INT) /* comment */ FROM foo
 SELECT a /* x */, b /* x */
 SELECT a /* x */ /* y */ /* z */, b /* k */ /* m */
 SELECT * FROM foo /* x */, bla /* x */
 SELECT 1 /* comment */ + 1
 SELECT 1 /* c1 */ + 2 /* c2 */
+SELECT 1 /* c1 */ + /* c2 */ 2 /* c3 */
 SELECT 1 /* c1 */ + 2 /* c2 */ + 3 /* c3 */
 SELECT 1 /* c1 */ + 2 /* c2 */, 3 /* c3 */
 SELECT x FROM a.b.c /* x */, e.f.g /* x */
 SELECT FOO(x /* c */) /* FOO */, b /* b */
 SELECT FOO(x /* c1 */ + y /* c2 */ + BLA(5 /* c3 */)) FROM (VALUES (1 /* c4 */, "test" /* c5 */)) /* c6 */
+INSERT INTO foo SELECT * FROM bar /* comment */
+/* c */ WITH x AS (SELECT 1) SELECT * FROM x
 SELECT a FROM x WHERE a COLLATE 'utf8_general_ci' = 'b'
 SELECT x AS INTO FROM bla
 SELECT * INTO newevent FROM event
 SELECT * INTO TEMPORARY newevent FROM event
 SELECT * INTO UNLOGGED newevent FROM event
 ALTER TABLE integers ADD COLUMN k INT
+ALTER TABLE integers ADD COLUMN k INT FIRST
+ALTER TABLE integers ADD COLUMN k INT AFTER m
 ALTER TABLE integers ADD COLUMN IF NOT EXISTS k INT
 ALTER TABLE IF EXISTS integers ADD COLUMN k INT
 ALTER TABLE integers ADD COLUMN l INT DEFAULT 10
 ALTER TABLE measurements ADD COLUMN mtime TIMESTAMPTZ DEFAULT NOW()
 ALTER TABLE integers DROP COLUMN k
+ALTER TABLE integers DROP PRIMARY KEY
 ALTER TABLE integers DROP COLUMN IF EXISTS k
 ALTER TABLE integers DROP COLUMN k CASCADE
 ALTER TABLE integers ALTER COLUMN i TYPE VARCHAR
@@ -696,6 +756,7 @@ ALTER TABLE orders DROP PARTITION(dt = '2014-05-14', country = 'IN')
 ALTER TABLE orders DROP IF EXISTS PARTITION(dt = '2014-05-14', country = 'IN')
 ALTER TABLE orders DROP PARTITION(dt = '2014-05-14', country = 'IN'), PARTITION(dt = '2014-05-15', country = 'IN')
 ALTER TABLE mydataset.mytable DELETE WHERE x = 1
+ALTER TABLE table1 MODIFY COLUMN name1 SET TAG foo='bar'
 SELECT div.a FROM test_table AS div
 WITH view AS (SELECT 1 AS x) SELECT * FROM view
 ARRAY<STRUCT<INT, DOUBLE, ARRAY<INT>>>
@@ -707,6 +768,7 @@ STRUCT<VARCHAR(10)>("bla")
 STRUCT<INT>(5)
 STRUCT<DATE>("2011-05-05")
 STRUCT<x INT, y TEXT>(1, t.str_col)
+STRUCT<int INT>
 SELECT CAST(NULL AS ARRAY<INT>) IS NULL AS array_is_null
 ALTER TABLE "schema"."tablename" ADD CONSTRAINT "CHK_Name" CHECK (NOT "IdDwh" IS NULL AND "IdDwh" <> (0))
 ALTER TABLE persons ADD CONSTRAINT persons_pk PRIMARY KEY (first_name, last_name)
@@ -714,10 +776,47 @@ ALTER TABLE pets ADD CONSTRAINT pets_persons_fk FOREIGN KEY (owner_first_name, o
 ALTER TABLE pets ADD CONSTRAINT pets_name_not_cute_chk CHECK (LENGTH(name) < 20)
 ALTER TABLE people10m ADD CONSTRAINT dateWithinRange CHECK (birthDate > '1900-01-01')
 ALTER TABLE people10m ADD CONSTRAINT validIds CHECK (id > 1 AND id < 99999999) ENFORCED
+ALTER TABLE s_ut ADD CONSTRAINT s_ut_uq UNIQUE hajo
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo PRIMARY KEY (x, y) NOT ENFORCED DEFERRABLE INITIALLY DEFERRED NORELY
 ALTER TABLE baa ADD CONSTRAINT boo FOREIGN KEY (x, y) REFERENCES persons ON UPDATE NO ACTION ON DELETE NO ACTION MATCH FULL
 ALTER TABLE a ADD PRIMARY KEY (x, y) NOT ENFORCED
 ALTER TABLE a ADD FOREIGN KEY (x, y) REFERENCES bla
+SELECT partition FROM a
 SELECT end FROM a
 SELECT id FROM b.a AS a QUALIFY ROW_NUMBER() OVER (PARTITION BY br ORDER BY sadf DESC) = 1
+SELECT LEFT.FOO FROM BLA AS LEFT
+SELECT RIGHT.FOO FROM BLA AS RIGHT
+SELECT LEFT FROM LEFT LEFT JOIN RIGHT RIGHT JOIN LEFT
+SELECT * FROM x WHERE name ILIKE ANY XXX('a', 'b')
+SELECT * FROM x WHERE name LIKE ANY XXX('a', 'b')
+a OVERLAPS b
+GRANT INSERT ON foo TO bla
+PRAGMA quick_check
+PRAGMA QUICK_CHECK(0)
+PRAGMA QUICK_CHECK('sqlite_master')
+PRAGMA schema.quick_check
+PRAGMA schema.QUICK_CHECK(0)
+PRAGMA schema.QUICK_CHECK('sqlite_master')
+PRAGMA synchronous = 2
+PRAGMA synchronous = FULL
+PRAGMA memory_limit = '1GB'
+PRAGMA schema.synchronous = 2
+PRAGMA schema.synchronous = FULL
+PRAGMA schema.memory_limit = '1GB'
+JSON_OBJECT()
+JSON_OBJECT('key1': 1, 'key2': TRUE)
+JSON_OBJECT('id': '5', 'fld1': 'bla', 'fld2': 'bar')
+JSON_OBJECT('x': NULL, 'y': 1 NULL ON NULL)
+JSON_OBJECT('x': NULL, 'y': 1 WITH UNIQUE KEYS)
+JSON_OBJECT('x': NULL, 'y': 1 ABSENT ON NULL WITH UNIQUE KEYS)
+JSON_OBJECT('x': 1 RETURNING VARCHAR(100))
+JSON_OBJECT('x': 1 RETURNING VARBINARY FORMAT JSON ENCODING UTF8)
+SELECT if.x
+SELECT NEXT VALUE FOR db.schema.sequence_name
+SELECT NEXT VALUE FOR db.schema.sequence_name OVER (ORDER BY foo), col
+SELECT PERCENTILE_CONT(x, 0.5) OVER ()
+SELECT PERCENTILE_CONT(x, 0.5 RESPECT NULLS) OVER ()
+SELECT PERCENTILE_CONT(x, 0.5 IGNORE NULLS) OVER ()
+WITH my_cte AS (SELECT 'a' AS desc) SELECT desc AS description FROM my_cte
+WITH my_cte AS (SELECT 'a' AS asc) SELECT asc AS description FROM my_cte
