@@ -1,3 +1,4 @@
+import datetime
 import unittest
 from datetime import date
 
@@ -57,6 +58,7 @@ class TestExecutor(unittest.TestCase):
 
     def test_py_dialect(self):
         self.assertEqual(Python().generate(parse_one("'x '''")), r"'x \''")
+        self.assertEqual(Python().generate(parse_one("MAP([1], [2])")), "MAP([1], [2])")
 
     def test_optimized_tpch(self):
         for i, (sql, optimized) in enumerate(self.sqls[:20], start=1):
@@ -401,6 +403,16 @@ class TestExecutor(unittest.TestCase):
             ],
         )
 
+        table1_view = exp.Select().select("id", "sub_type").from_("table1").subquery()
+        select_from_sub_query = exp.Select().select("id AS id_alias", "sub_type").from_(table1_view)
+        expression = exp.Select().select("*").from_("cte1").with_("cte1", as_=select_from_sub_query)
+
+        schema = {"table1": {"id": "str", "sub_type": "str"}}
+        executed = execute(expression, tables={t: [] for t in schema}, schema=schema)
+
+        self.assertEqual(executed.rows, [])
+        self.assertEqual(executed.columns, ("id_alias", "sub_type"))
+
     def test_correlated_count(self):
         tables = {
             "parts": [{"pnum": 0, "qoh": 1}],
@@ -503,6 +515,8 @@ class TestExecutor(unittest.TestCase):
         self.assertEqual(result.rows, [(3,)])
 
     def test_scalar_functions(self):
+        now = datetime.datetime.now()
+
         for sql, expected in [
             ("CONCAT('a', 'b')", "ab"),
             ("CONCAT('a', NULL)", None),
@@ -553,12 +567,41 @@ class TestExecutor(unittest.TestCase):
             ("IF(false, 1, 0)", 0),
             ("CASE WHEN 0 = 1 THEN 'foo' ELSE 'bar' END", "bar"),
             ("CAST('2022-01-01' AS DATE) + INTERVAL '1' DAY", date(2022, 1, 2)),
+            ("INTERVAL '1' week", datetime.timedelta(weeks=1)),
             ("1 IN (1, 2, 3)", True),
             ("1 IN (2, 3)", False),
             ("NULL IS NULL", True),
             ("NULL IS NOT NULL", False),
             ("NULL = NULL", None),
             ("NULL <> NULL", None),
+            ("YEAR(CURRENT_TIMESTAMP)", now.year),
+            ("MONTH(CURRENT_TIME)", now.month),
+            ("DAY(CURRENT_DATETIME())", now.day),
+            ("YEAR(CURRENT_DATE())", now.year),
+            ("MONTH(CURRENT_DATE())", now.month),
+            ("DAY(CURRENT_DATE())", now.day),
+            ("YEAR(CURRENT_TIMESTAMP) + 1", now.year + 1),
+            (
+                "YEAR(CURRENT_TIMESTAMP) IN (YEAR(CURRENT_TIMESTAMP) + 1, YEAR(CURRENT_TIMESTAMP) * 10)",
+                False,
+            ),
+            ("YEAR(CURRENT_TIMESTAMP) = (YEAR(CURRENT_TIMESTAMP))", True),
+            ("YEAR(CURRENT_TIMESTAMP) <> (YEAR(CURRENT_TIMESTAMP))", False),
+            ("YEAR(CURRENT_DATE()) + 1", now.year + 1),
+            (
+                "YEAR(CURRENT_DATE()) IN (YEAR(CURRENT_DATE()) + 1, YEAR(CURRENT_DATE()) * 10)",
+                False,
+            ),
+            ("YEAR(CURRENT_DATE()) = (YEAR(CURRENT_DATE()))", True),
+            ("YEAR(CURRENT_DATE()) <> (YEAR(CURRENT_DATE()))", False),
+            ("1::bool", True),
+            ("0::bool", False),
+            ("MAP(['a'], [1]).a", 1),
+            ("MAP()", {}),
+            ("STRFTIME('%j', '2023-03-23 15:00:00')", "082"),
+            ("STRFTIME('%j', NULL)", None),
+            ("DATESTRTODATE('2022-01-01')", date(2022, 1, 1)),
+            ("TIMESTRTOTIME('2022-01-01')", datetime.datetime(2022, 1, 1)),
         ]:
             with self.subTest(sql):
                 result = execute(f"SELECT {sql}")
@@ -566,5 +609,27 @@ class TestExecutor(unittest.TestCase):
 
     def test_case_sensitivity(self):
         result = execute("SELECT A AS A FROM X", tables={"x": [{"a": 1}]})
+        self.assertEqual(result.columns, ("a",))
+        self.assertEqual(result.rows, [(1,)])
+
+        result = execute('SELECT A AS "A" FROM X', tables={"x": [{"a": 1}]})
         self.assertEqual(result.columns, ("A",))
         self.assertEqual(result.rows, [(1,)])
+
+    def test_nested_table_reference(self):
+        tables = {
+            "some_catalog": {
+                "some_schema": {
+                    "some_table": [
+                        {"id": 1, "price": 1.0},
+                        {"id": 2, "price": 2.0},
+                        {"id": 3, "price": 3.0},
+                    ]
+                }
+            }
+        }
+
+        result = execute("SELECT * FROM some_catalog.some_schema.some_table s", tables=tables)
+
+        self.assertEqual(result.columns, ("id", "price"))
+        self.assertEqual(result.rows, [(1, 1.0), (2, 2.0), (3, 3.0)])
