@@ -6,6 +6,8 @@ from sqlglot import alias, exp, parse_one
 
 
 class TestExpressions(unittest.TestCase):
+    maxDiff = None
+
     def test_arg_key(self):
         self.assertEqual(parse_one("sum(1)").find(exp.Literal).arg_key, "this")
 
@@ -13,6 +15,20 @@ class TestExpressions(unittest.TestCase):
         self.assertEqual(parse_one("x(1)").find(exp.Literal).depth, 1)
 
     def test_eq(self):
+        self.assertEqual(exp.to_identifier("a"), exp.to_identifier("A"))
+
+        self.assertEqual(
+            exp.Column(table=exp.to_identifier("b"), this=exp.to_identifier("b")),
+            exp.Column(this=exp.to_identifier("b"), table=exp.to_identifier("b")),
+        )
+
+        self.assertEqual(exp.to_identifier("a", quoted=True), exp.to_identifier("A"))
+        self.assertNotEqual(exp.to_identifier("A", quoted=True), exp.to_identifier("A"))
+        self.assertNotEqual(
+            exp.to_identifier("A", quoted=True), exp.to_identifier("a", quoted=True)
+        )
+        self.assertNotEqual(parse_one("'x'"), parse_one("'X'"))
+        self.assertNotEqual(parse_one("'1'"), parse_one("1"))
         self.assertEqual(parse_one("`a`", read="hive"), parse_one('"a"'))
         self.assertEqual(parse_one("`a`", read="hive"), parse_one('"a"  '))
         self.assertEqual(parse_one("`a`.b", read="hive"), parse_one('"a"."b"'))
@@ -91,6 +107,37 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(column.parent_select, exp.Select)
         self.assertIsNone(column.find_ancestor(exp.Join))
 
+    def test_to_dot(self):
+        column = parse_one('a.b.c."d".e.f').find(exp.Column)
+        dot = column.to_dot()
+
+        self.assertEqual(dot.sql(), 'a.b.c."d".e.f')
+
+        self.assertEqual(
+            dot,
+            exp.Dot(
+                this=exp.Dot(
+                    this=exp.Dot(
+                        this=exp.Dot(
+                            this=exp.Dot(
+                                this=exp.to_identifier("a"),
+                                expression=exp.to_identifier("b"),
+                            ),
+                            expression=exp.to_identifier("c"),
+                        ),
+                        expression=exp.to_identifier("d", quoted=True),
+                    ),
+                    expression=exp.to_identifier("e"),
+                ),
+                expression=exp.to_identifier("f"),
+            ),
+        )
+
+    def test_root(self):
+        ast = parse_one("select * from (select a from x)")
+        self.assertIs(ast, ast.root())
+        self.assertIs(ast, ast.find(exp.Column).root())
+
     def test_alias_or_name(self):
         expression = parse_one(
             "SELECT a, b AS B, c + d AS e, *, 'zz', 'zz' AS z FROM foo as bar, baz"
@@ -100,8 +147,8 @@ class TestExpressions(unittest.TestCase):
             ["a", "B", "e", "*", "zz", "z"],
         )
         self.assertEqual(
-            [e.alias_or_name for e in expression.args["from"].expressions],
-            ["bar", "baz"],
+            {e.alias_or_name for e in expression.find_all(exp.Table)},
+            {"bar", "baz"},
         )
 
         expression = parse_one(
@@ -117,11 +164,16 @@ class TestExpressions(unittest.TestCase):
             ["first", "second"],
         )
 
+        self.assertEqual("first", expression.args["from"].alias_or_name)
         self.assertEqual(
-            [e.alias_or_name for e in expression.args["from"].expressions],
-            ["first", "second", "third"],
+            [e.alias_or_name for e in expression.args["joins"]],
+            ["second", "third"],
         )
 
+        self.assertEqual(parse_one("x.*").name, "*")
+        self.assertEqual(parse_one("NULL").name, "NULL")
+        self.assertEqual(parse_one("a.b.c").name, "c")
+
     def test_table_name(self):
         self.assertEqual(exp.table_name(parse_one("a", into=exp.Table)), "a")
         self.assertEqual(exp.table_name(parse_one("a.b", into=exp.Table)), "a.b")
@@ -134,39 +186,40 @@ class TestExpressions(unittest.TestCase):
     def test_replace_tables(self):
         self.assertEqual(
             exp.replace_tables(
-                parse_one("select * from a AS a join b join c.a join d.a join e.a"),
+                parse_one("select * from a AS a, b, c.a, d.a cross join e.a"),
                 {"a": "a1", "b": "b.a", "c.a": "c.a2", "d.a": "d2"},
             ).sql(),
-            "SELECT * FROM a1 AS a JOIN b.a JOIN c.a2 JOIN d2 JOIN e.a",
+            "SELECT * FROM a1 AS a, b.a, c.a2, d2 CROSS JOIN e.a",
         )
 
     def test_replace_placeholders(self):
         self.assertEqual(
             exp.replace_placeholders(
-                parse_one("select * from :tbl1 JOIN :tbl2 ON :col1 = :col2 WHERE :col3 > 100"),
-                tbl1="foo",
-                tbl2="bar",
-                col1="a",
-                col2="b",
-                col3="c",
+                parse_one("select * from :tbl1 JOIN :tbl2 ON :col1 = :str1 WHERE :col2 > :int1"),
+                tbl1=exp.to_identifier("foo"),
+                tbl2=exp.to_identifier("bar"),
+                col1=exp.to_identifier("a"),
+                col2=exp.to_identifier("c"),
+                str1="b",
+                int1=100,
             ).sql(),
-            "SELECT * FROM foo JOIN bar ON a = b WHERE c > 100",
+            "SELECT * FROM foo JOIN bar ON a = 'b' WHERE c > 100",
         )
         self.assertEqual(
             exp.replace_placeholders(
-                parse_one("select * from ? JOIN ? ON ? = ? WHERE ? > 100"),
-                "foo",
-                "bar",
-                "a",
+                parse_one("select * from ? JOIN ? ON ? = ? WHERE ? = 'bla'"),
+                exp.to_identifier("foo"),
+                exp.to_identifier("bar"),
+                exp.to_identifier("a"),
                 "b",
-                "c",
+                "bla",
             ).sql(),
-            "SELECT * FROM foo JOIN bar ON a = b WHERE c > 100",
+            "SELECT * FROM foo JOIN bar ON a = 'b' WHERE 'bla' = 'bla'",
         )
         self.assertEqual(
             exp.replace_placeholders(
                 parse_one("select * from ? WHERE ? > 100"),
-                "foo",
+                exp.to_identifier("foo"),
             ).sql(),
             "SELECT * FROM foo WHERE ? > 100",
         )
@@ -178,23 +231,25 @@ class TestExpressions(unittest.TestCase):
         )
         self.assertEqual(
             exp.replace_placeholders(
-                parse_one("select * from (SELECT :col1 FROM ?) WHERE :col2 > 100"),
-                "tbl1",
-                "tbl2",
+                parse_one("select * from (SELECT :col1 FROM ?) WHERE :col2 > ?"),
+                exp.to_identifier("tbl1"),
+                100,
                 "tbl3",
-                col1="a",
-                col2="b",
+                col1=exp.to_identifier("a"),
+                col2=exp.to_identifier("b"),
                 col3="c",
             ).sql(),
             "SELECT * FROM (SELECT a FROM tbl1) WHERE b > 100",
         )
 
     def test_function_building(self):
-        self.assertEqual(exp.func("bla", 1, "foo").sql(), "BLA(1, 'foo')")
+        self.assertEqual(exp.func("max", 1).sql(), "MAX(1)")
+        self.assertEqual(exp.func("max", 1, 2).sql(), "MAX(1, 2)")
+        self.assertEqual(exp.func("bla", 1, "foo").sql(), "BLA(1, foo)")
         self.assertEqual(exp.func("COUNT", exp.Star()).sql(), "COUNT(*)")
         self.assertEqual(exp.func("bloo").sql(), "BLOO()")
         self.assertEqual(
-            exp.func("locate", "x", "xo", dialect="hive").sql("hive"), "LOCATE('x', 'xo')"
+            exp.func("locate", "'x'", "'xo'", dialect="hive").sql("hive"), "LOCATE('x', 'xo')"
         )
 
         self.assertIsInstance(exp.func("instr", "x", "b", dialect="mysql"), exp.StrPosition)
@@ -243,6 +298,9 @@ class TestExpressions(unittest.TestCase):
         expression = parse_one("SELECT a, b FROM x")
         self.assertEqual([s.sql() for s in expression.selects], ["a", "b"])
 
+        expression = parse_one("(SELECT a, b FROM x)")
+        self.assertEqual([s.sql() for s in expression.selects], ["a", "b"])
+
     def test_alias_column_names(self):
         expression = parse_one("SELECT * FROM (SELECT * FROM x) AS y")
         subquery = expression.find(exp.Subquery)
@@ -291,6 +349,7 @@ class TestExpressions(unittest.TestCase):
         self.assertEqual(parse_one("x + y * 2").sql(), "x + y * 2")
         self.assertEqual(parse_one('select "x"').sql(dialect="hive", pretty=True), "SELECT\n  `x`")
         self.assertEqual(parse_one("X + y").sql(identify=True, normalize=True), '"x" + "y"')
+        self.assertEqual(parse_one('"X" + Y').sql(identify=True, normalize=True), '"X" + "y"')
         self.assertEqual(parse_one("SUM(X)").sql(identify=True, normalize=True), 'SUM("x")')
 
     def test_transform_with_arguments(self):
@@ -402,6 +461,8 @@ class TestExpressions(unittest.TestCase):
         )
 
     def test_functions(self):
+        self.assertIsInstance(parse_one("x LIKE ANY (y)"), exp.Like)
+        self.assertIsInstance(parse_one("x ILIKE ANY (y)"), exp.ILike)
         self.assertIsInstance(parse_one("ABS(a)"), exp.Abs)
         self.assertIsInstance(parse_one("APPROX_DISTINCT(a)"), exp.ApproxDistinct)
         self.assertIsInstance(parse_one("ARRAY(a)"), exp.Array)
@@ -409,22 +470,28 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("ARRAY_CONTAINS(a, 'a')"), exp.ArrayContains)
         self.assertIsInstance(parse_one("ARRAY_SIZE(a)"), exp.ArraySize)
         self.assertIsInstance(parse_one("AVG(a)"), exp.Avg)
+        self.assertIsInstance(parse_one("BEGIN DEFERRED TRANSACTION"), exp.Transaction)
         self.assertIsInstance(parse_one("CEIL(a)"), exp.Ceil)
         self.assertIsInstance(parse_one("CEILING(a)"), exp.Ceil)
         self.assertIsInstance(parse_one("COALESCE(a, b)"), exp.Coalesce)
+        self.assertIsInstance(parse_one("COMMIT"), exp.Commit)
         self.assertIsInstance(parse_one("COUNT(a)"), exp.Count)
+        self.assertIsInstance(parse_one("COUNT_IF(a > 0)"), exp.CountIf)
         self.assertIsInstance(parse_one("DATE_ADD(a, 1)"), exp.DateAdd)
         self.assertIsInstance(parse_one("DATE_DIFF(a, 2)"), exp.DateDiff)
         self.assertIsInstance(parse_one("DATE_STR_TO_DATE(a)"), exp.DateStrToDate)
         self.assertIsInstance(parse_one("DAY(a)"), exp.Day)
         self.assertIsInstance(parse_one("EXP(a)"), exp.Exp)
         self.assertIsInstance(parse_one("FLOOR(a)"), exp.Floor)
+        self.assertIsInstance(parse_one("GENERATE_SERIES(a, b, c)"), exp.GenerateSeries)
+        self.assertIsInstance(parse_one("GLOB(x, y)"), exp.Glob)
         self.assertIsInstance(parse_one("GREATEST(a, b)"), exp.Greatest)
         self.assertIsInstance(parse_one("IF(a, b, c)"), exp.If)
         self.assertIsInstance(parse_one("INITCAP(a)"), exp.Initcap)
         self.assertIsInstance(parse_one("JSON_EXTRACT(a, '$.name')"), exp.JSONExtract)
         self.assertIsInstance(parse_one("JSON_EXTRACT_SCALAR(a, '$.name')"), exp.JSONExtractScalar)
         self.assertIsInstance(parse_one("LEAST(a, b)"), exp.Least)
+        self.assertIsInstance(parse_one("LIKE(x, y)"), exp.Like)
         self.assertIsInstance(parse_one("LN(a)"), exp.Ln)
         self.assertIsInstance(parse_one("LOG10(a)"), exp.Log10)
         self.assertIsInstance(parse_one("MAX(a)"), exp.Max)
@@ -436,6 +503,7 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("QUANTILE(a, 0.90)"), exp.Quantile)
         self.assertIsInstance(parse_one("REGEXP_LIKE(a, 'test')"), exp.RegexpLike)
         self.assertIsInstance(parse_one("REGEXP_SPLIT(a, 'test')"), exp.RegexpSplit)
+        self.assertIsInstance(parse_one("ROLLBACK"), exp.Rollback)
         self.assertIsInstance(parse_one("ROUND(a)"), exp.Round)
         self.assertIsInstance(parse_one("ROUND(a, 2)"), exp.Round)
         self.assertIsInstance(parse_one("SPLIT(a, 'test')"), exp.Split)
@@ -463,29 +531,29 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("VARIANCE(a)"), exp.Variance)
         self.assertIsInstance(parse_one("VARIANCE_POP(a)"), exp.VariancePop)
         self.assertIsInstance(parse_one("YEAR(a)"), exp.Year)
-        self.assertIsInstance(parse_one("BEGIN DEFERRED TRANSACTION"), exp.Transaction)
-        self.assertIsInstance(parse_one("COMMIT"), exp.Commit)
-        self.assertIsInstance(parse_one("ROLLBACK"), exp.Rollback)
-        self.assertIsInstance(parse_one("GENERATE_SERIES(a, b, c)"), exp.GenerateSeries)
+        self.assertIsInstance(parse_one("HLL(a)"), exp.Hll)
+        self.assertIsInstance(parse_one("ARRAY(time, foo)"), exp.Array)
+        self.assertIsInstance(parse_one("STANDARD_HASH('hello', 'sha256')"), exp.StandardHash)
 
     def test_column(self):
-        dot = parse_one("a.b.c")
-        column = dot.this
-        self.assertEqual(column.table, "a")
-        self.assertEqual(column.name, "b")
-        self.assertEqual(dot.text("expression"), "c")
+        column = parse_one("a.b.c.d")
+        self.assertEqual(column.catalog, "a")
+        self.assertEqual(column.db, "b")
+        self.assertEqual(column.table, "c")
+        self.assertEqual(column.name, "d")
 
         column = parse_one("a")
         self.assertEqual(column.name, "a")
         self.assertEqual(column.table, "")
 
-        fields = parse_one("a.b.c.d")
+        fields = parse_one("a.b.c.d.e")
         self.assertIsInstance(fields, exp.Dot)
-        self.assertEqual(fields.text("expression"), "d")
-        self.assertEqual(fields.this.text("expression"), "c")
+        self.assertEqual(fields.text("expression"), "e")
         column = fields.find(exp.Column)
-        self.assertEqual(column.name, "b")
-        self.assertEqual(column.table, "a")
+        self.assertEqual(column.name, "d")
+        self.assertEqual(column.table, "c")
+        self.assertEqual(column.db, "b")
+        self.assertEqual(column.catalog, "a")
 
         column = parse_one("a[0].b")
         self.assertIsInstance(column, exp.Dot)
@@ -499,10 +567,11 @@ class TestExpressions(unittest.TestCase):
         self.assertEqual(column.table, "a")
 
         self.assertIsInstance(parse_one("*"), exp.Star)
+        self.assertEqual(exp.column("a", table="b", db="c", catalog="d"), exp.to_column("d.c.b.a"))
 
     def test_text(self):
-        column = parse_one("a.b.c")
-        self.assertEqual(column.text("expression"), "c")
+        column = parse_one("a.b.c.d.e")
+        self.assertEqual(column.text("expression"), "e")
         self.assertEqual(column.text("y"), "")
         self.assertEqual(parse_one("select * from x.y").find(exp.Table).text("db"), "x")
         self.assertEqual(parse_one("select *").name, "")
@@ -543,7 +612,6 @@ class TestExpressions(unittest.TestCase):
                     "FORMAT": "parquet",
                     "PARTITIONED_BY": (exp.to_identifier("a"), exp.to_identifier("b")),
                     "custom": 1,
-                    "TABLE_FORMAT": exp.to_identifier("test_format"),
                     "ENGINE": None,
                     "COLLATE": True,
                 }
@@ -555,7 +623,6 @@ class TestExpressions(unittest.TestCase):
                         this=exp.Tuple(expressions=[exp.to_identifier("a"), exp.to_identifier("b")])
                     ),
                     exp.Property(this=exp.Literal.string("custom"), value=exp.Literal.number(1)),
-                    exp.TableFormatProperty(this=exp.to_identifier("test_format")),
                     exp.EngineProperty(this=exp.null()),
                     exp.CollateProperty(this=exp.true()),
                 ]
@@ -659,10 +726,6 @@ FROM foo""",
         self.assertEqual(catalog_db_and_table.args.get("catalog"), exp.to_identifier("catalog"))
         with self.assertRaises(ValueError):
             exp.to_table(1)
-        empty_string = exp.to_table("")
-        self.assertEqual(empty_string.name, "")
-        self.assertIsNone(table_only.args.get("db"))
-        self.assertIsNone(table_only.args.get("catalog"))
 
     def test_to_column(self):
         column_only = exp.to_column("column_name")
@@ -697,7 +760,7 @@ FROM foo""",
                 "t",
                 {"a": exp.DataType.build("TEXT"), "b": exp.DataType.build("TEXT")},
             ).sql(),
-            "(VALUES (CAST(1 AS TEXT), CAST(2 AS TEXT)), (3, 4)) AS t(a, b)",
+            "(VALUES (1, 2), (3, 4)) AS t(a, b)",
         )
         with self.assertRaises(ValueError):
             exp.values([(1, 2), (3, 4)], columns=["a"])
@@ -760,3 +823,36 @@ FROM foo""",
             exp.rename_table("t1", "t2").sql(),
             "ALTER TABLE t1 RENAME TO t2",
         )
+
+    def test_is_star(self):
+        assert parse_one("*").is_star
+        assert parse_one("foo.*").is_star
+        assert parse_one("SELECT * FROM foo").is_star
+        assert parse_one("(SELECT * FROM foo)").is_star
+        assert parse_one("SELECT *, 1 FROM foo").is_star
+        assert parse_one("SELECT foo.* FROM foo").is_star
+        assert parse_one("SELECT * EXCEPT (a, b) FROM foo").is_star
+        assert parse_one("SELECT foo.* EXCEPT (foo.a, foo.b) FROM foo").is_star
+        assert parse_one("SELECT * REPLACE (a AS b, b AS C)").is_star
+        assert parse_one("SELECT * EXCEPT (a, b) REPLACE (a AS b, b AS C)").is_star
+        assert parse_one("SELECT * INTO newevent FROM event").is_star
+        assert parse_one("SELECT * FROM foo UNION SELECT * FROM bar").is_star
+        assert parse_one("SELECT * FROM bla UNION SELECT 1 AS x").is_star
+        assert parse_one("SELECT 1 AS x UNION SELECT * FROM bla").is_star
+        assert parse_one("SELECT 1 AS x UNION SELECT 1 AS x UNION SELECT * FROM foo").is_star
+
+    def test_set_metadata(self):
+        ast = parse_one("SELECT foo.col FROM foo")
+
+        self.assertIsNone(ast._meta)
+
+        # calling ast.meta would lazily instantiate self._meta
+        self.assertEqual(ast.meta, {})
+        self.assertEqual(ast._meta, {})
+
+        ast.meta["some_meta_key"] = "some_meta_value"
+        self.assertEqual(ast.meta.get("some_meta_key"), "some_meta_value")
+        self.assertEqual(ast.meta.get("some_other_meta_key"), None)
+
+        ast.meta["some_other_meta_key"] = "some_other_meta_value"
+        self.assertEqual(ast.meta.get("some_other_meta_key"), "some_other_meta_value")
