@@ -1,4 +1,5 @@
 import unittest
+from concurrent.futures import ProcessPoolExecutor, as_completed
 from functools import partial
 
 import duckdb
@@ -11,6 +12,7 @@ from sqlglot.optimizer.annotate_types import annotate_types
 from sqlglot.optimizer.scope import build_scope, traverse_scope, walk_in_scope
 from sqlglot.schema import MappingSchema
 from tests.helpers import (
+    TPCDS_SCHEMA,
     TPCH_SCHEMA,
     load_sql_fixture_pairs,
     load_sql_fixtures,
@@ -18,11 +20,35 @@ from tests.helpers import (
 )
 
 
+def parse_and_optimize(func, sql, read_dialect, **kwargs):
+    return func(parse_one(sql, read=read_dialect), **kwargs)
+
+
+def qualify_columns(expression, **kwargs):
+    expression = optimizer.qualify.qualify(
+        expression, infer_schema=True, validate_qualify_columns=False, identify=False, **kwargs
+    )
+    return expression
+
+
+def pushdown_projections(expression, **kwargs):
+    expression = optimizer.qualify_tables.qualify_tables(expression)
+    expression = optimizer.qualify_columns.qualify_columns(expression, infer_schema=True, **kwargs)
+    expression = optimizer.pushdown_projections.pushdown_projections(expression, **kwargs)
+    return expression
+
+
+def normalize(expression, **kwargs):
+    expression = optimizer.normalize.normalize(expression, dnf=False)
+    return optimizer.simplify.simplify(expression)
+
+
 class TestOptimizer(unittest.TestCase):
     maxDiff = None
 
     @classmethod
     def setUpClass(cls):
+        sqlglot.schema = MappingSchema()
         cls.conn = duckdb.connect()
         cls.conn.execute(
             """
@@ -73,30 +99,44 @@ class TestOptimizer(unittest.TestCase):
             },
         }
 
-    def check_file(self, file, func, pretty=False, execute=False, **kwargs):
-        for i, (meta, sql, expected) in enumerate(
-            load_sql_fixture_pairs(f"optimizer/{file}.sql"), start=1
-        ):
-            title = meta.get("title") or f"{i}, {sql}"
-            dialect = meta.get("dialect")
-            leave_tables_isolated = meta.get("leave_tables_isolated")
+    def check_file(self, file, func, pretty=False, execute=False, set_dialect=False, **kwargs):
+        with ProcessPoolExecutor() as pool:
+            results = {}
 
-            func_kwargs = {**kwargs}
-            if leave_tables_isolated is not None:
-                func_kwargs["leave_tables_isolated"] = string_to_bool(leave_tables_isolated)
+            for i, (meta, sql, expected) in enumerate(
+                load_sql_fixture_pairs(f"optimizer/{file}.sql"), start=1
+            ):
+                title = meta.get("title") or f"{i}, {sql}"
+                dialect = meta.get("dialect")
+                leave_tables_isolated = meta.get("leave_tables_isolated")
+
+                func_kwargs = {**kwargs}
+                if leave_tables_isolated is not None:
+                    func_kwargs["leave_tables_isolated"] = string_to_bool(leave_tables_isolated)
+
+                if set_dialect and dialect:
+                    func_kwargs["dialect"] = dialect
+
+                future = pool.submit(parse_and_optimize, func, sql, dialect, **func_kwargs)
+                results[future] = (
+                    sql,
+                    title,
+                    expected,
+                    dialect,
+                    execute if meta.get("execute") is None else False,
+                )
+
+        for future in as_completed(results):
+            optimized = future.result()
+            sql, title, expected, dialect, execute = results[future]
 
             with self.subTest(title):
-                optimized = func(parse_one(sql, read=dialect), **func_kwargs)
                 self.assertEqual(
                     expected,
                     optimized.sql(pretty=pretty, dialect=dialect),
                 )
 
-            should_execute = meta.get("execute")
-            if should_execute is None:
-                should_execute = execute
-
-            if string_to_bool(should_execute):
+            if string_to_bool(execute):
                 with self.subTest(f"(execute) {title}"):
                     df1 = self.conn.execute(
                         sqlglot.transpile(sql, read=dialect, write="duckdb")[0]
@@ -105,13 +145,24 @@ class TestOptimizer(unittest.TestCase):
                     assert_frame_equal(df1, df2)
 
     def test_optimize(self):
+        self.assertEqual(optimizer.optimize("x = 1 + 1", identify=None).sql(), "x = 2")
+
         schema = {
             "x": {"a": "INT", "b": "INT"},
             "y": {"b": "INT", "c": "INT"},
             "z": {"a": "INT", "c": "INT"},
+            "u": {"f": "INT", "g": "INT", "h": "TEXT"},
         }
 
-        self.check_file("optimizer", optimizer.optimize, pretty=True, execute=True, schema=schema)
+        self.check_file(
+            "optimizer",
+            optimizer.optimize,
+            infer_schema=True,
+            pretty=True,
+            execute=True,
+            schema=schema,
+            set_dialect=True,
+        )
 
     def test_isolate_table_selects(self):
         self.check_file(
@@ -137,25 +188,28 @@ class TestOptimizer(unittest.TestCase):
             "(x AND y) OR (x AND z)",
         )
 
-        self.check_file(
-            "normalize",
-            optimizer.normalize.normalize,
+        self.assertEqual(
+            optimizer.normalize.normalize(
+                parse_one("x AND (y OR z)"),
+            ).sql(),
+            "x AND (y OR z)",
         )
 
+        self.check_file("normalize", normalize)
+
     def test_qualify_columns(self):
-        def qualify_columns(expression, **kwargs):
-            expression = optimizer.qualify_tables.qualify_tables(expression)
-            expression = optimizer.qualify_columns.qualify_columns(expression, **kwargs)
-            return expression
+        self.assertEqual(
+            optimizer.qualify_columns.qualify_columns(
+                parse_one("select y from x"),
+                schema={},
+                infer_schema=False,
+            ).sql(),
+            "SELECT y AS y FROM x",
+        )
 
         self.check_file("qualify_columns", qualify_columns, execute=True, schema=self.schema)
 
     def test_qualify_columns__with_invisible(self):
-        def qualify_columns(expression, **kwargs):
-            expression = optimizer.qualify_tables.qualify_tables(expression)
-            expression = optimizer.qualify_columns.qualify_columns(expression, **kwargs)
-            return expression
-
         schema = MappingSchema(self.schema, {"x": {"a"}, "y": {"b"}, "z": {"b"}})
         self.check_file("qualify_columns__with_invisible", qualify_columns, schema=schema)
 
@@ -163,23 +217,28 @@ class TestOptimizer(unittest.TestCase):
         for sql in load_sql_fixtures("optimizer/qualify_columns__invalid.sql"):
             with self.subTest(sql):
                 with self.assertRaises((OptimizeError, SchemaError)):
-                    optimizer.qualify_columns.qualify_columns(parse_one(sql), schema=self.schema)
+                    expression = optimizer.qualify_columns.qualify_columns(
+                        parse_one(sql), schema=self.schema
+                    )
+                    optimizer.qualify_columns.validate_qualify_columns(expression)
 
-    def test_lower_identities(self):
-        self.check_file("lower_identities", optimizer.lower_identities.lower_identities)
+    def test_normalize_identifiers(self):
+        self.check_file(
+            "normalize_identifiers",
+            optimizer.normalize_identifiers.normalize_identifiers,
+            set_dialect=True,
+        )
 
     def test_pushdown_projection(self):
-        def pushdown_projections(expression, **kwargs):
-            expression = optimizer.qualify_tables.qualify_tables(expression)
-            expression = optimizer.qualify_columns.qualify_columns(expression, **kwargs)
-            expression = optimizer.pushdown_projections.pushdown_projections(expression)
-            return expression
-
         self.check_file("pushdown_projections", pushdown_projections, schema=self.schema)
 
     def test_simplify(self):
         self.check_file("simplify", optimizer.simplify.simplify)
 
+        expression = parse_one("TRUE AND TRUE AND TRUE")
+        self.assertEqual(exp.true(), optimizer.simplify.simplify(expression))
+        self.assertEqual(exp.true(), optimizer.simplify.simplify(expression.this))
+
     def test_unnest_subqueries(self):
         self.check_file(
             "unnest_subqueries",
@@ -190,10 +249,20 @@ class TestOptimizer(unittest.TestCase):
     def test_pushdown_predicates(self):
         self.check_file("pushdown_predicates", optimizer.pushdown_predicates.pushdown_predicates)
 
-    def test_expand_multi_table_selects(self):
-        self.check_file(
-            "expand_multi_table_selects",
-            optimizer.expand_multi_table_selects.expand_multi_table_selects,
+    def test_expand_alias_refs(self):
+        # check order of lateral expansion with no schema
+        self.assertEqual(
+            optimizer.optimize("SELECT a + 1 AS d, d + 1 AS e FROM x WHERE e > 1 GROUP BY e").sql(),
+            'SELECT "x"."a" + 1 AS "d", "x"."a" + 2 AS "e" FROM "x" AS "x" WHERE "x"."a" + 2 > 1 GROUP BY "x"."a" + 2',
+        )
+
+        self.assertEqual(
+            optimizer.qualify_columns.qualify_columns(
+                parse_one("SELECT CAST(x AS INT) AS y FROM z AS z"),
+                schema={"l": {"c": "int"}},
+                infer_schema=False,
+            ).sql(),
+            "SELECT CAST(x AS INT) AS y FROM z AS z",
         )
 
     def test_optimize_joins(self):
@@ -235,8 +304,8 @@ class TestOptimizer(unittest.TestCase):
         optimize = partial(
             optimizer.optimize,
             rules=[
-                optimizer.qualify_tables.qualify_tables,
-                optimizer.qualify_columns.qualify_columns,
+                optimizer.qualify.qualify,
+                optimizer.qualify_columns.quote_identifiers,
                 annotate_types,
                 optimizer.canonicalize.canonicalize,
             ],
@@ -246,6 +315,9 @@ class TestOptimizer(unittest.TestCase):
     def test_tpch(self):
         self.check_file("tpc-h/tpc-h", optimizer.optimize, schema=TPCH_SCHEMA, pretty=True)
 
+    def test_tpcds(self):
+        self.check_file("tpc-ds/tpc-ds", optimizer.optimize, schema=TPCDS_SCHEMA, pretty=True)
+
     def test_file_schema(self):
         expression = parse_one(
             """
@@ -348,11 +420,16 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
         self.assertEqual(expression.type.this, exp.DataType.Type.TIMESTAMPTZ)
         self.assertEqual(expression.this.type.this, exp.DataType.Type.VARCHAR)
         self.assertEqual(expression.args["to"].type.this, exp.DataType.Type.TIMESTAMPTZ)
-        self.assertEqual(expression.args["to"].expressions[0].type.this, exp.DataType.Type.INT)
+        self.assertEqual(expression.args["to"].expressions[0].this.type.this, exp.DataType.Type.INT)
 
         expression = annotate_types(parse_one("ARRAY(1)::ARRAY<INT>"))
         self.assertEqual(expression.type, parse_one("ARRAY<INT>", into=exp.DataType))
 
+        expression = annotate_types(parse_one("CAST(x AS INTERVAL)"))
+        self.assertEqual(expression.type.this, exp.DataType.Type.INTERVAL)
+        self.assertEqual(expression.this.type.this, exp.DataType.Type.UNKNOWN)
+        self.assertEqual(expression.args["to"].type.this, exp.DataType.Type.INTERVAL)
+
     def test_cache_annotation(self):
         expression = annotate_types(
             parse_one("CACHE LAZY TABLE x OPTIONS('storageLevel' = 'value') AS SELECT 1")
@@ -369,6 +446,12 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
         self.assertEqual(expression.right.this.left.type.this, exp.DataType.Type.INT)
         self.assertEqual(expression.right.this.right.type.this, exp.DataType.Type.INT)
 
+    def test_lateral_annotation(self):
+        expression = optimizer.optimize(
+            parse_one("SELECT c FROM (select 1 a) as x LATERAL VIEW EXPLODE (a) AS c")
+        ).expressions[0]
+        self.assertEqual(expression.type.this, exp.DataType.Type.INT)
+
     def test_derived_tables_column_annotation(self):
         schema = {"x": {"cola": "INT"}, "y": {"cola": "FLOAT"}}
         sql = """
@@ -391,7 +474,7 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
             expression.expressions[0].type.this, exp.DataType.Type.FLOAT
         )  # a.cola AS cola
 
-        addition_alias = expression.args["from"].expressions[0].this.expressions[0]
+        addition_alias = expression.args["from"].this.this.expressions[0]
         self.assertEqual(
             addition_alias.type.this, exp.DataType.Type.FLOAT
         )  # x.cola + y.cola AS cola
@@ -502,6 +585,8 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
             concat_expr.right.expressions[0].type.this, exp.DataType.Type.VARCHAR
         )  # x.cola (arg)
 
+        annotate_types(parse_one("select x from y lateral view explode(y) as x")).expressions[0]
+
     def test_null_annotation(self):
         expression = annotate_types(parse_one("SELECT NULL + 2 AS col")).expressions[0].this
         self.assertEqual(expression.left.type.this, exp.DataType.Type.NULL)
@@ -554,6 +639,16 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
             )
             self.assertEqual(expression.expressions[0].type.this, target_type)
 
+    def test_concat_annotation(self):
+        expression = annotate_types(parse_one("CONCAT('A', 'B')"))
+        self.assertEqual(expression.type.this, exp.DataType.Type.VARCHAR)
+
+    def test_root_subquery_annotation(self):
+        expression = annotate_types(parse_one("(SELECT 1, 2 FROM x) LIMIT 0"))
+        self.assertIsInstance(expression, exp.Subquery)
+        self.assertEqual(exp.DataType.Type.INT, expression.selects[0].type.this)
+        self.assertEqual(exp.DataType.Type.INT, expression.selects[1].type.this)
+
     def test_recursive_cte(self):
         query = parse_one(
             """
@@ -592,3 +687,30 @@ FROM READ_CSV('tests/fixtures/optimizer/tpc-h/nation.csv.gz', 'delimiter', '|')
             optimizer.optimize(parse_one("SELECT * FROM a"), schema=schema),
             parse_one('SELECT "a"."b c" AS "b c", "a"."d e" AS "d e" FROM "a" AS "a"'),
         )
+
+    def test_quotes(self):
+        schema = {
+            "example": {
+                '"source"': {
+                    "id": "text",
+                    '"name"': "text",
+                    '"payload"': "text",
+                }
+            }
+        }
+
+        expected = parse_one(
+            """
+            SELECT
+             "source"."ID" AS "ID",
+             "source"."name" AS "name",
+             "source"."payload" AS "payload"
+            FROM "EXAMPLE"."source" AS "source"
+            """,
+            read="snowflake",
+        ).sql(pretty=True, dialect="snowflake")
+
+        for func in (optimizer.qualify.qualify, optimizer.optimize):
+            source_query = parse_one('SELECT * FROM example."source"', read="snowflake")
+            transformed = func(source_query, dialect="snowflake", schema=schema)
+            self.assertEqual(transformed.sql(pretty=True, dialect="snowflake"), expected)
