@@ -20,6 +20,9 @@ class TestTranspile(unittest.TestCase):
         self.assertEqual(transpile(sql, **kwargs)[0], target)
 
     def test_alias(self):
+        self.assertEqual(transpile("SELECT SUM(y) KEEP")[0], "SELECT SUM(y) AS KEEP")
+        self.assertEqual(transpile("SELECT 1 overwrite")[0], "SELECT 1 AS overwrite")
+        self.assertEqual(transpile("SELECT 1 is")[0], "SELECT 1 AS is")
         self.assertEqual(transpile("SELECT 1 current_time")[0], "SELECT 1 AS current_time")
         self.assertEqual(
             transpile("SELECT 1 current_timestamp")[0], "SELECT 1 AS current_timestamp"
@@ -28,6 +31,11 @@ class TestTranspile(unittest.TestCase):
         self.assertEqual(transpile("SELECT 1 current_datetime")[0], "SELECT 1 AS current_datetime")
         self.assertEqual(transpile("SELECT 1 row")[0], "SELECT 1 AS row")
 
+        self.assertEqual(
+            transpile("SELECT 1 FROM a.b.table1 t UNPIVOT((c3) FOR c4 IN (a, b))")[0],
+            "SELECT 1 FROM a.b.table1 AS t UNPIVOT((c3) FOR c4 IN (a, b))",
+        )
+
         for key in ("union", "over", "from", "join"):
             with self.subTest(f"alias {key}"):
                 self.validate(f"SELECT x AS {key}", f"SELECT x AS {key}")
@@ -82,6 +90,7 @@ class TestTranspile(unittest.TestCase):
         self.validate("SELECT 3>=3", "SELECT 3 >= 3")
 
     def test_comments(self):
+        self.validate("SELECT 1 /*/2 */", "SELECT 1 /* /2 */")
         self.validate("SELECT */*comment*/", "SELECT * /* comment */")
         self.validate(
             "SELECT * FROM table /*comment 1*/ /*comment 2*/",
@@ -195,11 +204,87 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
             read="mysql",
             pretty=True,
         )
+        self.validate(
+            """
+            SELECT a FROM b
+            WHERE foo
+            -- comment 1
+            AND bar
+            -- comment 2
+            AND bla;
+            """,
+            "SELECT a FROM b WHERE foo AND /* comment 1 */ bar AND /* comment 2 */ bla",
+        )
+        self.validate(
+            """
+            SELECT a FROM b WHERE foo
+            -- comment 1
+            """,
+            "SELECT a FROM b WHERE foo /* comment 1 */",
+        )
+        self.validate(
+            """
+            select a
+            -- from
+            from b
+            -- where
+            where foo
+            -- comment 1
+            and bar
+            -- comment 2
+            and bla
+            """,
+            """SELECT
+  a
+/* from */
+FROM b
+/* where */
+WHERE
+  foo /* comment 1 */ AND bar AND bla /* comment 2 */""",
+            pretty=True,
+        )
+        self.validate(
+            """
+            -- test
+            WITH v AS (
+              SELECT
+                1 AS literal
+            )
+            SELECT
+              *
+            FROM v
+            """,
+            """/* test */
+WITH v AS (
+  SELECT
+    1 AS literal
+)
+SELECT
+  *
+FROM v""",
+            pretty=True,
+        )
+        self.validate(
+            "(/* 1 */ 1 ) /* 2 */",
+            "(1) /* 1 */ /* 2 */",
+        )
+        self.validate(
+            "select * from t where not a in (23) /*test*/ and b in (14)",
+            "SELECT * FROM t WHERE NOT a IN (23) /* test */ AND b IN (14)",
+        )
+        self.validate(
+            "select * from t where a in (23) /*test*/ and b in (14)",
+            "SELECT * FROM t WHERE a IN (23) /* test */ AND b IN (14)",
+        )
+        self.validate(
+            "select * from t where ((condition = 1)/*test*/)",
+            "SELECT * FROM t WHERE ((condition = 1) /* test */)",
+        )
 
     def test_types(self):
-        self.validate("INT x", "CAST(x AS INT)")
-        self.validate("VARCHAR x y", "CAST(x AS VARCHAR) AS y")
-        self.validate("STRING x y", "CAST(x AS TEXT) AS y")
+        self.validate("INT 1", "CAST(1 AS INT)")
+        self.validate("VARCHAR 'x' y", "CAST('x' AS VARCHAR) AS y")
+        self.validate("STRING 'x' y", "CAST('x' AS TEXT) AS y")
         self.validate("x::INT", "CAST(x AS INT)")
         self.validate("x::INTEGER", "CAST(x AS INT)")
         self.validate("x::INT y", "CAST(x AS INT) AS y")
@@ -216,7 +301,7 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
         self.validate("a NOT BETWEEN b AND c", "NOT a BETWEEN b AND c")
         self.validate("a NOT IN (1, 2)", "NOT a IN (1, 2)")
         self.validate("a IS NOT NULL", "NOT a IS NULL")
-        self.validate("a LIKE TEXT y", "a LIKE CAST(y AS TEXT)")
+        self.validate("a LIKE TEXT 'y'", "a LIKE CAST('y' AS TEXT)")
 
     def test_extract(self):
         self.validate(
@@ -256,9 +341,6 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
         )
         self.validate("SELECT IF(a > 1, 1) FROM foo", "SELECT CASE WHEN a > 1 THEN 1 END FROM foo")
 
-    def test_ignore_nulls(self):
-        self.validate("SELECT COUNT(x RESPECT NULLS)", "SELECT COUNT(x)")
-
     def test_with(self):
         self.validate(
             "WITH a AS (SELECT 1) WITH b AS (SELECT 2) SELECT *",
@@ -283,7 +365,6 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
             "ALTER TABLE integers ADD k INTEGER",
             "ALTER TABLE integers ADD COLUMN k INT",
         )
-        self.validate("ALTER TABLE integers DROP k", "ALTER TABLE integers DROP COLUMN k")
         self.validate(
             "ALTER TABLE integers ALTER i SET DATA TYPE VARCHAR",
             "ALTER TABLE integers ALTER COLUMN i TYPE VARCHAR",
@@ -294,6 +375,11 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
         )
 
     def test_time(self):
+        self.validate("INTERVAL '1 day'", "INTERVAL '1' day")
+        self.validate("INTERVAL '1 days' * 5", "INTERVAL '1' days * 5")
+        self.validate("5 * INTERVAL '1 day'", "5 * INTERVAL '1' day")
+        self.validate("INTERVAL 1 day", "INTERVAL '1' day")
+        self.validate("INTERVAL 2 months", "INTERVAL '2' months")
         self.validate("TIMESTAMP '2020-01-01'", "CAST('2020-01-01' AS TIMESTAMP)")
         self.validate("TIMESTAMP WITH TIME ZONE '2020-01-01'", "CAST('2020-01-01' AS TIMESTAMPTZ)")
         self.validate(
@@ -410,7 +496,7 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
         self.validate("UNIX_TO_STR(123, 'y')", "FROM_UNIXTIME(123, 'y')", write="spark")
         self.validate(
             "UNIX_TO_TIME(123)",
-            "FROM_UNIXTIME(123)",
+            "CAST(FROM_UNIXTIME(123) AS TIMESTAMP)",
             write="spark",
         )
         self.validate(
@@ -426,6 +512,13 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
         mock_logger.warning.assert_any_call("Applying array index offset (%s)", 1)
         mock_logger.warning.assert_any_call("Applying array index offset (%s)", -1)
 
+        self.validate("x[x - 1]", "x[x - 1]", write="presto", identity=False)
+        self.validate(
+            "x[array_size(y) - 1]", "x[CARDINALITY(y) - 1 + 1]", write="presto", identity=False
+        )
+        self.validate("x[3 - 1]", "x[3]", write="presto", identity=False)
+        self.validate("MAP(a, b)[0]", "MAP(a, b)[0]", write="presto", identity=False)
+
     def test_identify_lambda(self):
         self.validate("x(y -> y)", 'X("y" -> "y")', identify=True)
 
@@ -492,31 +585,24 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
 
         with self.assertRaises(ParseError) as ctx:
             transpile(invalid, error_level=ErrorLevel.IMMEDIATE)
+
         self.assertEqual(str(ctx.exception), expected_messages[0])
         self.assertEqual(ctx.exception.errors[0], expected_errors[0])
 
         with self.assertRaises(ParseError) as ctx:
             transpile(invalid, error_level=ErrorLevel.RAISE)
+
         self.assertEqual(str(ctx.exception), "\n\n".join(expected_messages))
         self.assertEqual(ctx.exception.errors, expected_errors)
 
         more_than_max_errors = "(((("
         expected_messages = (
-            "Expecting ). Line 1, Col: 4.\n  (((\033[4m(\033[0m\n\n"
             "Required keyword: 'this' missing for <class 'sqlglot.expressions.Paren'>. Line 1, Col: 4.\n  (((\033[4m(\033[0m\n\n"
             "Expecting ). Line 1, Col: 4.\n  (((\033[4m(\033[0m\n\n"
+            "Expecting ). Line 1, Col: 4.\n  (((\033[4m(\033[0m\n\n"
             "... and 2 more"
         )
         expected_errors = [
-            {
-                "description": "Expecting )",
-                "line": 1,
-                "col": 4,
-                "start_context": "(((",
-                "highlight": "(",
-                "end_context": "",
-                "into_expression": None,
-            },
             {
                 "description": "Required keyword: 'this' missing for <class 'sqlglot.expressions.Paren'>",
                 "line": 1,
@@ -526,12 +612,22 @@ FROM bar /* comment 5 */, tbl /*          comment 6 */""",
                 "end_context": "",
                 "into_expression": None,
             },
+            {
+                "description": "Expecting )",
+                "line": 1,
+                "col": 4,
+                "start_context": "(((",
+                "highlight": "(",
+                "end_context": "",
+                "into_expression": None,
+            },
         ]
         # Also expect three trailing structured errors that match the first
-        expected_errors += [expected_errors[0]] * 3
+        expected_errors += [expected_errors[1]] * 3
 
         with self.assertRaises(ParseError) as ctx:
             transpile(more_than_max_errors, error_level=ErrorLevel.RAISE)
+
         self.assertEqual(str(ctx.exception), expected_messages)
         self.assertEqual(ctx.exception.errors, expected_errors)
 
